      subroutine isex03( iblock, ndim, ni, nj, nk, nvirt, target,
     +                   vartyp, dist, nblocks, nfaces, maxnsubf,
     +                   nsubfaces, subfaces)
c ======================================================================
c
c     programmer  Ronald van Nooyen
c
c     version     2.4    date  07-04-1997 Kees K: multiblock
c     version     2.3    date  31-10-1995 Kees K: adaption for periodic
c                         bc's removed
c     version     2.2    date  09-10-1995 Kees K: bug fixed
c     version     2.1    date  04-08-1994 Michiel K: Adapted for
c                        periodic bc's.
c     version     2.0    date  27-12-1994 code cleanup
c     version     1.0    date  17-08-1994
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     ISNaS Extrapolate vartyp in TARGET DIST positions outward
c     for points that do not correspond to internal boundaries
c     Points that lie beyond the boundary in two or more
c     directions are ALWAYS filled by extrapolation. In case of periodic
c     boundaries the virtual cells are filled in ISEXVPE3. On the edges
c     and vertices ISEX03 fills the virtual cells when at least one
c     boundary condition is non-periodic.
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iblock, maxnsubf, nblocks, nfaces
      integer nsubfaces(1:nblocks, 1:nfaces)
      integer subfaces(1:nblocks, 1:nfaces, 1:maxnsubf, *)
      integer ndim, ni, nj, nk, nvirt, vartyp, dist
      double precision target
     +     (1-nvirt:ni+1+nvirt,1-nvirt:nj+1+nvirt,1-nvirt:nk+1+nvirt)

c     dist      i    The distance between two neighbour points around a
c                    given boundary
c     iblock    i    Block sequence number
c     maxnsubf  i    Maximum number of subfaces encountered on all subfaces
c     nblocks   i    Number of blocks
c     ndim      i    Dimension of space (2 or 3)
c     nfaces    i    Number of faces
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     nsubfaces i    Array containing the number of subfaces of each face,
c                    see array itopol for an extended description
c     nvirt     i    Number of virtual unknowns
c     subfaces  i    reserved
c     target    i    Target point
c     vartyp    i    Variable to be interpolated/extrapolated
c                    vartyp: 1 for V1, 2 for V2 and 3 for pressure,
c                    0 for vertex (for ndim=2)
c                    or
c                    vartyp: 1 for V1, 2 for V2 3, for V3 and
c                    4 for pressure, 0 for vertex  (for ndim=3)
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ep1, ep2, ep3, locdist, iface, isubface
      integer varea(1:2,1:3), carea(1:2,1:3),
     +        ntdir(1:3), ntsig(1:3),
     +        area(1:2,1:3),ndir,nsig,
     +        tp1, tp2, edir,
     +        w1p(1:3), w2p(1:3), w3p(1:3),
     +        w12p(1:3), w23p(1:3), w13p(1:3),
     +        xp(1:3), yp(1:3), zp(1:3),
     +        tdir1, tdir2, betad

c     area      Summation index bounds
c     betad     Direction index, usually the direction perpendicular to the
c               integration direction of the boundary integral.
c               For example betad in local point (1,0) is 1 and in (0,1): 2
c     carea     Index bounds on set of grid cells adjacent to
c               this boundary
c     edir      Edge direction
c     ep1       Auxiliary index
c     ep2       Auxiliary index
c     ep3       Auxiliary index
c     iface     Face number
c     isubface  Subface number
c     locdist
c     ndir      Direction of normal vector on this side
c     nsig      Sign of inner product of the basis vector
c               for the above mentioned internal direction
c               and the normal
c     ntdir     Contains the ksi-alpha indices of the normal and
c               tangential directions
c               ntdir(1) refers to the normal direction
c               ntdir(2) refers to the first tangential direction
c               ntdir(3) refers to the second tangential direction
c     ntsig     Gives the direction of the normal or tangential vectors
c               with respect to the ksi co-ordinate axis
c               The normal is supposed to be the outward normal and
c               the tangential directions are taken into a counter
c               clockwise way
c               ntsig(1) refers to the normal direction
c               ntsig(2) refers to the first tangential direction
c               ntsig(3) refers to the second tangential direction
c     tdir1     Direction of first tangent vector on current side
c     tdir2     Direction of second tangent vector on current side
c     tp1       Tangential point index in tangential direction 1
c     tp2       Tangential point index in tangential direction 2
c     varea     Area of arrays
c     w12p      Auxiliary array for multilinear interpolation
c     w13p      Auxiliary array for multilinear interpolation
c     w1p       Auxiliary array for multilinear interpolation
c     w23p      Auxiliary array for multilinear interpolation
c     w2p       Auxiliary array for multilinear interpolation
c     w3p       Auxiliary array for multilinear interpolation
c     xp        Vector with indices of the point in
c               extrapolation formula furthest from the edge
c     yp        Vector with indices of the point in
c               extrapolation formula closest to the edge
c     zp        Target point of the extrapolation formula
c **********************************************************************
c
c                       I/O
c
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISBNBVI     get information on current side
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c     locdist  = 0
c     Note: assume values known for all points within the domain
c     Repeat:
c        For all sides:
c           If at an external boundary then
c           Extrapolate to points adjacent to currently known points
c        Extrapolate to adjacent unknowns on cube edges or in corners
c        locdist = locdist+1
c     Until values are known upto distance dist beyond the domain
c
c
c **********************************************************************
c
c                       KEYWORDS
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
c     --- loop over extrapolation distances

      do 100 locdist=0,dist-1

c     --- loop over all planes

         do 110 iface = 1, nfaces
            do isubface = 1, nsubfaces(iblock, iface)
               if ( subfaces(iblock, iface, isubface, 5).eq.0 ) then

c              --- get information on extent and direction of a side

                  call isbnbvi(ndim,ni,nj,nk,iface,ntdir,ntsig,
     +                         carea,varea)

                  ndir   = ntdir(1)
                  tdir1  = ntdir(2)
                  tdir2  = ntdir(3)
                  nsig   = ntsig(1)

c                 --- adjust carea and varea to reflect the work already done

                  varea(1,ndir) = varea(1,ndir)+nsig*locdist
                  varea(2,ndir) = varea(2,ndir)+nsig*locdist
                  carea(1,ndir) = carea(1,ndir)+nsig*locdist
                  carea(2,ndir) = carea(2,ndir)+nsig*locdist

                  varea(1,tdir1) = varea(1,tdir1)-locdist
                  varea(2,tdir1) = varea(2,tdir1)+locdist
                  carea(1,tdir1) = carea(1,tdir1)-locdist
                  carea(2,tdir1) = carea(2,tdir1)+locdist

                  varea(1,tdir2) = varea(1,tdir2)-locdist
                  varea(2,tdir2) = varea(2,tdir2)+locdist
                  carea(1,tdir2) = carea(1,tdir2)-locdist
                  carea(2,tdir2) = carea(2,tdir2)+locdist

c                 --- prepare to extrapolate

c                     --- area:
c                     line of known values in the domain closest to the
c                     unknown values

                  if (vartyp.eq.0) then

                     area(1,ndir)  = varea(1,ndir)
                     area(2,ndir)  = varea(2,ndir)
                     area(1,tdir1) = varea(1,tdir1)
                     area(2,tdir1) = varea(2,tdir1)
                     area(1,tdir2) = varea(1,tdir2)
                     area(2,tdir2) = varea(2,tdir2)

                  else if (vartyp.eq.ndir) then

                     area(1,ndir)  = varea(1,ndir)
                     area(2,ndir)  = varea(2,ndir)
                     area(1,tdir1) = carea(1,tdir1)
                     area(2,tdir1) = carea(2,tdir1)
                     area(1,tdir2) = carea(1,tdir2)
                     area(2,tdir2) = carea(2,tdir2)

                  else if (vartyp.eq.tdir1) then

                     area(1,ndir)  = carea(1,ndir)
                     area(2,ndir)  = carea(2,ndir)
                     area(1,tdir1) = varea(1,tdir1)
                     area(2,tdir1) = varea(2,tdir1)
                     area(1,tdir2) = carea(1,tdir2)
                     area(2,tdir2) = carea(2,tdir2)

                  else if (vartyp.eq.tdir2) then

                     area(1,ndir)  = carea(1,ndir)
                     area(2,ndir)  = carea(2,ndir)
                     area(1,tdir1) = carea(1,tdir1)
                     area(2,tdir1) = carea(2,tdir1)
                     area(1,tdir2) = varea(1,tdir2)
                     area(2,tdir2) = varea(2,tdir2)

                  else if (vartyp.eq.ndim+1) then

                     area(1,ndir)  = carea(1,ndir)
                     area(2,ndir)  = carea(2,ndir)
                     area(1,tdir1) = carea(1,tdir1)
                     area(2,tdir1) = carea(2,tdir1)
                     area(1,tdir2) = carea(1,tdir2)
                     area(2,tdir2) = carea(2,tdir2)

                  end if

                  do 120 tp2=area(1,tdir2),area(2,tdir2)
                     do 130 tp1=area(1,tdir1),area(2,tdir1)

                        yp(tdir1) = tp1
                        yp(tdir2) = tp2
                        yp(ndir) = area(2,ndir)
                        xp(tdir1) = yp(tdir1)
                        xp(tdir2) = yp(tdir2)
                        xp(ndir) = yp(ndir)-nsig
                        zp(tdir1) = yp(tdir1)
                        zp(tdir2) = yp(tdir2)
                        zp(ndir) = yp(ndir)+nsig

c                       --- extrapolate to virtual cells

                        target(zp(1),zp(2),zp(3)) =
     +                     2d0*target(yp(1),yp(2),yp(3))-
     +                     target(xp(1),xp(2),xp(3))

130                  continue
120               continue

               end if
            end do

110      continue

c        --- now deal with edge and corner points

c            --- area:
c            line of known values in the domain closest to the edge

         if (vartyp.eq.0) then

            area(1,1) = 1-locdist
            area(2,1) = ni+1+locdist
            area(1,2) = 1-locdist
            area(2,2) = nj+1+locdist
            area(1,3) = 1-locdist
            area(2,3) = nk+1+locdist

         else if (vartyp.eq.1) then

            area(1,1) = 1-locdist
            area(2,1) = ni+1+locdist
            area(1,2) = 1-locdist
            area(2,2) = nj+locdist
            area(1,3) = 1-locdist
            area(2,3) = nk+locdist

         else if (vartyp.eq.2) then

            area(1,1) = 1-locdist
            area(2,1) = ni+locdist
            area(1,2) = 1-locdist
            area(2,2) = nj+1+locdist
            area(1,3) = 1-locdist
            area(2,3) = nk+locdist

         else if (vartyp.eq.3) then

            area(1,1) = 1-locdist
            area(2,1) = ni+locdist
            area(1,2) = 1-locdist
            area(2,2) = nj+locdist
            area(1,3) = 1-locdist
            area(2,3) = nk+1+locdist

         else if (vartyp.eq.ndim+1) then

            area(1,1) = 1-locdist
            area(2,1) = ni+locdist
            area(1,2) = 1-locdist
            area(2,2) = nj+locdist
            area(1,3) = 1-locdist
            area(2,3) = nk+locdist

         end if

c        --- Edge (bilinear extrapolation)

         do 200 ep3=0,2
            do 210 ep2=0,2
               do 220 ep1=0,2

c              --- Skip combinations of ep1,ep2,ep3 that
c                  do not correspond to an edge

                  if (
     +                 (ep1.ne.0 .and. ep2.ne.0 .and. ep3.eq.0) .or.
     +                 (ep1.ne.0 .and. ep3.ne.0 .and. ep2.eq.0) .or.
     +                 (ep3.ne.0 .and. ep2.ne.0 .and. ep1.eq.0) ) then

                  ntsig(1) = 0
                  ntsig(2) = 0
                  ntsig(3) = 0
                  if (ep1.ne.0) then
                     yp(1) = area(ep1,1)
                     ntsig(1) = 2*ep1-3
                  else
                     edir = 1
                  end if
                  if (ep2.ne.0) then
                     yp(2) = area(ep2,2)
                     ntsig(2) = 2*ep2-3
                  else
                     edir = 2
                  end if
                  if (ep3.ne.0) then
                     yp(3) = area(ep3,3)
                     ntsig(3) = 2*ep3-3
                  else
                     edir = 3
                  end if

                  do 300 tp1=area(1,edir),area(2,edir)

                     yp(edir) = tp1
                     do 310 betad=1,ndim
                        zp(betad) = yp(betad)+ntsig(betad)
                        xp(betad) = yp(betad)-ntsig(betad)
                        w1p(betad) = yp(betad)
                        w2p(betad) = yp(betad)
                        w3p(betad) = yp(betad)
310                  continue

                     betad = 1
                     w1p(betad) = yp(betad)-ntsig(betad)
                     betad = 2
                     w2p(betad) = yp(betad)-ntsig(betad)
                     betad = 3
                     w3p(betad) = yp(betad)-ntsig(betad)

c                    --- ALWAYS extrapolate to fill corners

c                             --- extrapolate to virtual cells

                           target(zp(1),zp(2),zp(3)) =
     +                          6d0*target(yp(1),yp(2),yp(3))-
     +                          2d0*(
     +                          target(w1p(1),w1p(2),w1p(3))+
     +                          target(w2p(1),w2p(2),w2p(3))+
     +                          target(w3p(1),w3p(2),w3p(3)))+
     +                          target(xp(1),xp(2),xp(3))

300               continue
                  end if
220          continue
210       continue
200    continue

c      --- Corner (trilinear extrapolation)

         do 400 ep3=1,2
            do 410 ep2=1,2
               do 420 ep1=1,2
                  ntsig(1) = 2*ep1-3
                  ntsig(2) = 2*ep2-3
                  ntsig(3) = 2*ep3-3
                  yp(1) = area(ep1,1)
                  yp(2) = area(ep2,2)
                  yp(3) = area(ep3,3)
                  do 430 betad=1,ndim
                     zp(betad) = yp(betad)+ntsig(betad)
                     xp(betad) = yp(betad)-ntsig(betad)
                     w1p(betad) = yp(betad)
                     w2p(betad) = yp(betad)
                     w3p(betad) = yp(betad)
                     w12p(betad) = yp(betad)
                     w23p(betad) = yp(betad)
                     w13p(betad) = yp(betad)
430               continue

                  betad = 1
                  w1p(betad) = yp(betad)-ntsig(betad)
                  w12p(betad) = yp(betad)-ntsig(betad)
                  w13p(betad) = yp(betad)-ntsig(betad)
                  betad = 2
                  w2p(betad) = yp(betad)-ntsig(betad)
                  w12p(betad) = yp(betad)-ntsig(betad)
                  w23p(betad) = yp(betad)-ntsig(betad)
                  betad = 3
                  w3p(betad) = yp(betad)-ntsig(betad)
                  w13p(betad) = yp(betad)-ntsig(betad)
                  w23p(betad) = yp(betad)-ntsig(betad)

c                 --- ALWAYS extrapolate to fill corners

c                          --- extrapolate to virtual cells

                        target(zp(1),zp(2),zp(3)) =
     +                       8d0*target(yp(1),yp(2),yp(3))-
     +                       4d0*(
     +                       target(w1p(1),w1p(2),w1p(3))+
     +                       target(w2p(1),w2p(2),w2p(3))+
     +                       target(w3p(1),w3p(2),w3p(3)))+
     +                       2d0*(
     +                       target(w12p(1),w12p(2),w12p(3))+
     +                       target(w13p(1),w13p(2),w13p(3))+
     +                       target(w23p(1),w23p(2),w23p(3))
     +                       )-
     +                       target(xp(1),xp(2),xp(3))

420            continue
410         continue
400      continue

100   continue

      end
