      subroutine isinnr2d ( coorl2, coorp, epsnr, maxiter, uci, vci )
c ======================================================================
c
c        programmer   Charles Moulinec
c        version      1.0  date 11-02-1998
c
c   copyright (c) 1998-1998  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Newton Raphson iteration, used for bilinear mapping 2D
c
c **********************************************************************
c
c                       KEYWORDS
c
c   2d
c   inverse_mapping
c   newton
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer maxiter
      double precision uci, vci, epsnr
      double precision coorl2(1:4,1:2),coorp(1:2)

c     coorl2    i    Array of coordinates of 4 surrounding points
c     coorp     i    Array of coordinates of the surrounded point
c     epsnr     i    Tolerance for bilinear metric coefficients
c     maxiter   i    Maximum number of iterations
c     uci       o    Local coordinate for the interpolation in the i-direction
c     vci       o    Local coordinate for the interpolation in the j-direction
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer iter, idir
      double precision fuv(1:2), dfuv(1:2,1:2),
     +                 duci, dvci, duuci, dvvci, detjac
      logical conv

c     conv      Logical variable indicating acquired convergence.
c     detjac    Determinant jacobian
c     dfuv      Jacobian of function fuv
c     duci      Finite difference of uci
c     duuci     Relative error of uci
c     dvci      Finite difference of vci
c     dvvci     Relative error of vci
c     fuv       Function of the two local coordinates u, v
c     idir      General loop variable for 1...ndim loop
c     iter      The current iteration number.
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISWARSUB  Print warnings
c **********************************************************************
c
c                       ERROR MESSAGES
c
c     1006     no convergence in computation of bilinear mapping
c **********************************************************************
c
c                       PSEUDO CODE
c
c      determine the functions of the two coordinates of the
c         intepolation point expressed in the coordinates of the four
c         surrounding points and the local coordinates uci, vci
c         used for interpolation.
c      compute jacobian
c      compute uci and vci with newton raphson iteration
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      uci = 0.5d0
      vci = 0.5d0

      conv = .false.
      iter = 1

100   if ( iter.le.maxiter .and. .not. conv ) then

         do idir = 1, 2

c        --- the two functions

            fuv(idir) =
     +         (coorl2(1,idir)-coorl2(2,idir)-
     +          coorl2(3,idir)+coorl2(4,idir))
     +         *uci*vci   +
     +         (-coorl2(1,idir)+coorl2(2,idir))
     +         *uci       +
     +         (-coorl2(1,idir)+coorl2(3,idir))
     +         *vci       +
     +         coorl2(1,idir) - coorp(idir)

c           --- Jacobian

            dfuv(idir,1) =
     +         (coorl2(1,idir)-coorl2(2,idir)-
     +          coorl2(3,idir)+coorl2(4,idir))
     +         *vci   +
     +         (-coorl2(1,idir)+coorl2(2,idir))
            dfuv(idir,2) =
     +         (coorl2(1,idir)-coorl2(2,idir)-
     +          coorl2(3,idir)+coorl2(4,idir))
     +         *uci   +
     +         (-coorl2(1,idir)+coorl2(3,idir))

         end do

c        --- Newton Raphson

         detjac =dfuv(1,1)*dfuv(2,2)-dfuv(1,2)*dfuv(2,1)

         duci= (-fuv(1)*dfuv(2,2)+fuv(2)*dfuv(1,2))/detjac
         dvci= (+fuv(1)*dfuv(2,1)-fuv(2)*dfuv(1,1))/detjac

         uci = uci + duci
         vci = vci + dvci

         if (abs(uci).lt.1.d-10) then
            duuci=1.d-10
            uci=0.d0
         else
            duuci=duci/uci
         end if

         if (abs(vci).lt.1.d-10) then
            dvvci=1.d-10
            vci=0.d0
         else
            dvvci=dvci/vci
         end if

         if ( abs(duuci).lt.epsnr .and. abs(dvvci).lt.epsnr )
     +        conv = .true.
         iter = iter + 1
         go to 100

      end if

      if ( .not. conv ) then
         call iseropen('isnton')
         call iswarsub(1006, 3, 0, 0)
         call iserclos('isnton')
      end if

      end
