      subroutine isbcs3 ( alphad, ni, nj, nk, prems, rhside,
     +                    locdx, ibipoint, ibindex, ibinfor, bndcon )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.0    date 25-05-1998
c
c   copyright (c) 1998-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c   Fill prescribed velocities in the term prems, which is used to build the
c   stress matrix. 3D WesBeek version
c   During the computation of the stress tensor the stress tensor must be
c   evaluated in integration points. In these integration points the
c   gradient of the velocity is required, since the stress tensor can be seen as
c   a linear combination of the velocity gradient.
c   The velocity gradient itself is expressed in terms of the velocity in
c   neighbouring points
c   If in some of these points the velocity is given, the corresponding value
c   can be substituted and this contribution is put into the right-hand side
c   In this subroutine prems and rhside are updated
c **********************************************************************
c
c                       KEYWORDS
c
c    boundary_conditions
c    discretization
c    momentum_equation
c    stress
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nk, ibipoint(*), ibinfor(*), alphad,
     +        locdx(3), ibindex(2,2,*)
      double precision rhside(ni+1,nj+1,nk+1,3),
     +                 prems(ni+1,nj+1,nk+1,3,6), bndcon(3,*)

c     alphad    i    Direction index of integration.
c                    Hence if a V1-cell is used, alphad = 1 and so on
c     bndcon    i    reserved
c     ibindex   i    ibindex(1,1,side)
c                                    first point index where information
c                                    is available on this side in
c                                    direction tdir (2D) or tdir(1) (3D)
c                    ibindex(2,1,side) last point index where information
c                                    is available on this side in
c                                    direction tdir (2D) or tdir(1) (3D
c                    ibindex(1,2,side)
c                                    first point index where information
c                                    is available on this side in
c                                    direction tdir or tdir(2) (3D)
c                    ibindex(2,2,side)
c                                    last point index where information
c                                    is available on this side in
c     ibinfor   i    Third part of infbnd
c     ibipoint  i    ipoint(side)
c                                 position of start of ibinfor information
c                                 for SIDE in ibinfor
c     locdx     i    Position of the integration point with respect to
c                    the centre of the integration cell
c                    The cell sizes are assumed to be 2 times 2, hence the
c                    distance of a midside point to a cell centre is always 1
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     prems    i/o   Intermediate matrix for building the stress term according
c                    to the Wesbeek discretization.
c     rhside   i/o   reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer iface, idir, ilow1, ihigh1, ilow2, ihigh2
      logical left

c     idir      General loop variable for 1...ndim loop
c     iface     Face number
c     ihigh1    First upper index of an array
c     ihigh2    Second upper index of an array
c     ilow1     First lower index of an array
c     ilow2     Second lower index of an array
c     left      If true the "left" or "under" face is used.
c               In R^2 this means faces 1 and 3
c               In R^3 faces 1, 3 and 5
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISBCS3N   Fill prescribed velocities in the stress term for "normal" cell
c     ISBCS3T   Fill prescribed velocities in the stress term for "tangent" cell
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c    Array prems(i,j,k,beta,gamma) contains the contribution of the velocity
c    to the stress tensor for a specific integration point with respect to the
c    V^alphad cell with center (i,j,k) (global coordinates)
c    beta (1,2,3) refers to the components of the Cartesian vector
c    Each integration point is surrounded by 6 points that are used to
c    express the gradient in terms of the velocity vector
c    Gamma refers to these six points
c    With respect to alphad = 1, the local shift of a gamma point compared
c    to an integration point is (-1,0,0), (1,0,0), (0,-1,0), ... , (0,0,1)
c    For alphad = 2 and 3 a cyclic permutation is used hence, for alphad
c    the sequence is (0,-1,0), (0,1,0), (0,0,-1), ...
c
c    If the cell center is on the boundary and the velocity is prescribed,
c    then the unknown itself is eliminated
c    hence in this subroutine we only consider points on the boundary that
c    correspond to cells with their center not on the boundary
c    There are two such cells:
c    Cells corresponding to a normal component at one cell distance of the
c    boundary and cells corresponding to a tangential component at half a
c    cell distance of the boundary
c
c    For a normal cell at one cell distance of the boundary, there is only
c    one point that is on the boundary
c    For an alphad cell this point corresponds to the integration point
c    in the alphad direction and in the direction of the boundary
c    Of this integration point only the surrounding point
c    in the alphad direction and in the direction of the boundary is on the
c    boundary
c    Hence the integration point must satisfy locdx(alphad) = 1 for an even
c    face and locdx(alphad) = -1 for an odd face.
c    The corresponding point at the boundary refers to gammad = 1 for an even
c    face and gammad = -1 for an odd face.
c
c    For a tangential cell we have much more points on the boundary
c    For example for face 1 (i=1), which implies alphad=2 or 3
c    we have an integration point on the boundary when locdx(1) = -1
c    For this integration point we have four neighbouring points on the
c    boundary and since an one-sided discretization is used, also the
c    point self is considered as a neighbouring point on the boundary
c    Points corresponding to locdx(1) = 0, are at a half cell distance of the
c    boundary, which implies that only one neighbouring point in the
c    alphad(=1) direction is positioned on the boundary
c    The same arguments can be used for the other faces
c
c    In this subroutine a preselection of the different cases is made, the
c    actual substitution takes place at one level lower
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- Loop over the faces

      do iface = 1, 6

c     --- Find positions in the boundary condition arrays

         ilow1  = ibindex(1,1,iface)
         ihigh1 = ibindex(2,1,iface)
         ilow2  = ibindex(1,2,iface)
         ihigh2 = ibindex(2,2,iface)

c        --- determine the type of face and express in direction and
c            "left or right"

         idir = (iface+1)/2
         left = mod(iface,2).eq.1
         if ( idir.eq.alphad .and. locdx(alphad).ne.0 ) then

c        --- Normal cell found, check if we are in the correct integration point

            if ( left .and. locdx(alphad).eq.-1 .or.
     +           .not. left .and. locdx(alphad).eq.1 ) then

c           --- integration point in the direction of the boundary
c               Call subroutine to adapt point for normal cell

              call isbcs3n ( iface, alphad, ni, nj, nk,
     +                       ibinfor(ibipoint(iface)), ilow1,
     +                       ihigh1, ilow2, ihigh2,
     +                       bndcon(1,ibipoint(iface)), prems, rhside )

            end if

         else if ( idir.ne.alphad ) then

c        --- Tangential cell found, check if we are in the correct
c            integration point

            if ( left .and. locdx(idir).ne.1 .or.
     +           .not. left .and. locdx(idir).ne.-1 ) then

c           --- integration point in the direction of the boundary
c               Call subroutine to adapt point for tangential cell

             call isbcs3t ( iface, alphad, locdx, ni, nj, nk,
     +                      ibinfor(ibipoint(iface)), ilow1,
     +                      ihigh1, ilow2, ihigh2,
     +                      bndcon(1,ibipoint(iface)), prems, rhside )

            end if

         end if

      end do

      end
