      subroutine isbnbvi( ndim , ni, nj, nk, nside, ntdir,
     +                    ntsig, carea, varea)
c======================================================================
c
c     programmer  Ronald van Nooyen
c
c     version     1.2     date  11-10-1995 sequence of tangential
c                                          directions changed Kees K
c     version     1.1     date  04-10-1994 sequence numbers of
c                                          boundaries changed Kees K
c
c     version     1.0     date  08-10-1993
c
c   copyright (c) 1994  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c**********************************************************************
c
c                 DESCRIPTION
c
c     The subroutine isbnbvi i.e.
c
c     ISnas Boundary Number to Basis Vector and Index information
c     ^^    ^        ^         ^     ^          ^
c     gives information on the relation between the ISNaS
c     basis vectors and the normal and tangential vectors on
c     a given boundary. Moreover it gives lower and upper bounds
c     on adjacent cell indices and on the indices of vertices on
c     the boundary.
c
c**********************************************************************
c
c                 KEYWORDS
c
c     boundary_conditions
c
c**********************************************************************
c
c                 INPUT / OUTPUT PARAMETERS
c
      implicit none

      integer maxdim
      parameter (maxdim = 3)
c
      integer ndim, ni, nj, nk, nside, ntdir(1:maxdim), ntsig(1:maxdim)
      integer varea(1:2,1:maxdim), carea(1:2, 1:maxdim)
c
c     carea     o     Index bounds on set of grid cells adjacent to
c                     this boundary
c     maxdim          Highest physical space dimension for which ISNaS can work
c     ndim      i     Dimension of space (2 or 3)
c     ni        i     Number of non-virtual cells in the i-direction (one block)
c     nj        i     Number of non-virtual cells in the j-direction (one block)
c     nk        i     Number of non-virtual cells in the k-direction (one block)
c     nside     i     Number of boundary for which we need the
c                     information
c     ntdir     o     Contains the ksi-alpha indices of the normal and
c                     tangential directions
c                     ntdir(1) refers to the normal direction
c                     ntdir(2) refers to the first tangential direction
c                     ntdir(3) refers to the second tangential direction
c     ntsig     o     Gives the direction of the normal or tangential vectors
c                     with respect to the ksi co-ordinate axis
c                     The normal is supposed to be the outward normal and
c                     the tangential directions are taken into a counter
c                     clockwise way
c                     ntsig(1) refers to the normal direction
c                     ntsig(2) refers to the first tangential direction
c                     ntsig(3) refers to the second tangential direction
c     varea     o     Area of arrays
c
c**********************************************************************
c
c                 COMMON BLOCKS
c
c
c**********************************************************************
c
c                 LOCAL PARAMETERS
c
      integer alphad, lowcell(1:maxdim), highcell(1:maxdim)
c
c     alphad      Direction index of integration.
c                 Hence if a V1-cell is used, alphad = 1 and so on
c     highcell    Upper bound on cell index in a given direction
c     lowcell     Lower bound on cell index in a given direction
c
c**********************************************************************
c
c                 SUBROUTINES CALLED
c
c**********************************************************************
c
c                 I/O
c
c**********************************************************************
c
c                 ERROR MESSAGES
c
c
c**********************************************************************
c
c                 PSEUDO CODE
c
c======================================================================
c
      if (ndim.eq.2) then

c     ---Initialize third array position in all arrays to
c        a sensible value

         ntdir(3) = 0
         ntsig(3) = 0
         carea(1,3) = 1
         carea(2,3) = 1
         varea(1,3) = 1
         varea(2,3) = 1

c     ---Set cell bounds

         lowcell(1)  = 1
         lowcell(2)  = 1
         highcell(1) = ni
         highcell(2) = nj


c     ---2D ISNaS numbers the domain boundaries as
c         follows:
c
c         +------4------+
c         |             |
c         |             |
c         |             |
c         1             2
c         |             |
c         |             |
c         |             |
c         +------3------+
c
c     --- Mathematical convention:
c         normal vector points outwards and is basis
c         vector e^1 in a right handed coordinate system, the tangent
c         vector is vector e^2 in this system.
c
c     --- ISNaS convention: the domain boundary curves directions
c         in the input file are such that when we walk in the curve
c         direction the
c         domain is at our left hand (i.e. we walk in the direction
c         given by the mathematical tangent vector)

         if (nside.eq.1) then

c     --- Left hand side:

c             normal points to the left, tangent points downwards,
c             basis vector 1 points to the right and vector 2 points
c             upwards, so

            ntdir(1) =  1
            ntdir(2) =  2
            ntsig(1) = -1
            ntsig(2) = -1

         else if (nside.eq.2) then

c     --- Right hand side:

c             normal points to the right, tangent points upwards,
c             basis vector 1 points to the right and vector 2 points
c             upwards, so


            ntdir(1) =  1
            ntdir(2) =  2
            ntsig(1) =  1
            ntsig(2) =  1

          else if (nside.eq.3) then

c     --- Lower side:

c             normal points downwards, tangent points to the right,
c             basis vector 1 points to the right and vector 2 points
c             upwards, so

            ntdir(1) =  2
            ntdir(2) =  1

            ntsig(1) = -1
            ntsig(2) =  1

         else if (nside.eq.4) then

c     --- Upper side:

c             normal points upwards, tangent points to the left,
c             basis vector 1 points to the right and vector 2 points
c             upwards, so

            ntdir(1) =  2
            ntdir(2) =  1
            ntsig(1) =  1
            ntsig(2) = -1


         end if

c     --- Set vertex and cell indices for normal direction

         if (ntsig(1).gt.0) then

c     --- vertex index in normal direction, nsig>0, so the
c         vertex has the index of a virtual cell beyond this
c         boundary

            carea(1,ntdir(1))   = highcell(ntdir(1))

            varea(1,ntdir(1))   = highcell(ntdir(1))+1

c     --- index in normal direction, so first index equals last index

            carea(2,ntdir(1))   = carea(1,ntdir(1))
            varea(2,ntdir(1))   = varea(1,ntdir(1))

         else

c     --- vertex index in normal direction, nsig<0, so the
c         cell and vertex have the index 1

            carea(1,ntdir(1))   = lowcell(ntdir(1))
            varea(1,ntdir(1))   = lowcell(ntdir(1))

c     --- index in normal direction, so first index equals last index

            carea(2,ntdir(1))   = carea(1,ntdir(1))
            varea(2,ntdir(1))   = varea(1,ntdir(1))
c
         end if

c     --- Set vertex and cell indices for tangential direction

         carea(1,ntdir(2)) = lowcell(ntdir(2))
         varea(1,ntdir(2)) = lowcell(ntdir(2))

         carea(2,ntdir(2)) = highcell(ntdir(2))

c     --- Last vertex in tangential direction has
c         the index of a virtual cell byond the boundary in the
c         tangential direction

         varea(2,ntdir(2)) = highcell(ntdir(2))+1

      else if (ndim.eq.3) then


c     --- Set cell bounds

         lowcell(1)  = 1
         lowcell(2)  = 1
         lowcell(3)  = 1
         highcell(1) = ni
         highcell(2) = nj
         highcell(3) = nk


c     --- Note that in 3D ISNaS numbers the domain boundaries as
c         follows: Vertices with fixed index

c         i = 1      lie on side 1,
c         i = ni+1   lie on side 2,
c         j = 1      lie on side 3,
c         j = nj+1   lie on side 4,
c         k = 1      lie on side 5,
c         k = nk+1   lie on side 6,

c     --- Mathematical convention:
c         normal vector points outwards and is basis
c         vector e^1 in a right handed coordinate system, the tangent
c         vectors are vectors e^2 and e^3 in this system.

c     --- ISNaS convention: none yet

c        --- initialize array ntdir and ntsig

         if (nside.eq.1 .or. nside.eq.2) then

c        --- ISNaS basis vector directions:
c            1 points forwards
c            2 points to the right
c            3 points upwards

            ntdir(1)    =  1
            ntdir(2)    =  2
            ntdir(3)    =  3

            if (nside.eq.1) then

c           --- Local basis vector system

c           Normal vector points backwards
c           First tangent vector points to the left
c           Second tangent vector points upwards

               ntsig(1) = -1
               ntsig(2) = -1
               ntsig(3) =  1

            else if (nside.eq.2) then

c           --- Local basis vector system

c           Normal vector points forwards
c           First tangent vector points to the right
c           Second tangent vector points upwards

               ntsig(1) =  1
               ntsig(2) =  1
               ntsig(3) =  1

            end if

         else if (nside.eq.3 .or. nside.eq.4) then

c        --- ISNaS basis vector directions:
c            1 points forwards
c            2 points to the right
c            3 points upwards

            ntdir(1)    =  2
            ntdir(2)    =  3
            ntdir(3)    =  1

            if (nside.eq.3) then

c           --- Local basis vector system

c           Normal vector points to the left
c           First tangent vector points forwards
c           Second tangent vector points upwards

               ntsig(1) = -1
               ntsig(2) = -1
               ntsig(3) =  1

            else if (nside.eq.4) then

c           --- Local basis vector system

c           Normal vector points to the right
c           First tangent vector points backwards
c           Second tangent vector points upwards

               ntsig(1) =  1
               ntsig(2) =  1
               ntsig(3) =  1

            end if

         else if (nside.eq.5 .or. nside.eq.6) then

c        --- ISNaS basis vector directions:
c            1 points forwards
c            2 points to the right
c            3 points upwards

            ntdir(1)    =  3
            ntdir(2)    =  1
            ntdir(3)    =  2

            if (nside.eq.5) then

c           --- Local basis vector system

c           Normal vector points downwards
c           First tangent vector points backwards
c           Second tangent vector points to the right

               ntsig(1) = -1
               ntsig(2) = -1
               ntsig(3) =  1

            else if (nside.eq.6) then

c           --- Local basis vector system

c           Normal vector points upwards
c           First tangent vector points forwards
c           Second tangent vector points to the right

               ntsig(1) =  1
               ntsig(2) =  1
               ntsig(3) =  1

            end if

         end if


c     --- Load carea and varea with default values

         do 100 alphad=1,ndim
            carea(1,alphad) = lowcell(alphad)
            carea(2,alphad) = highcell(alphad)

c        --- Lowest vertex index = lowest cell index

            varea(1,alphad) = lowcell(alphad)

c        --- Highest vertex index = highest cell index + 1

            varea(2,alphad) = highcell(alphad)+1

 100     continue

c        --- Set indices for normal direction

         if (mod(nside,2).eq.1) then

c        --- Low index case

            carea(2,ntdir(1)) = carea(1,ntdir(1))
            varea(2,ntdir(1)) = varea(1,ntdir(1))

         else

c        --- High index case

            carea(1,ntdir(1)) = carea(2,ntdir(1))
            varea(1,ntdir(1)) = varea(2,ntdir(1))

         end if

c        --- Indices for tangential directions already set

      end if

      end
