      subroutine isbwbp2 ( ni, nj, nvirtual, alphad,
     +                     ibinfor, ibipoint, bndcon, rhside,
     +                     bcgeo, adir, adiri, coor, axisym )
c ======================================================================
c
c        programmer     Duncan van der Heul
c        version 2.0    date 09-12-1999 Guus: extra parameters coor and axisym
c                                       (axi-synmmetric extension)
c        version 1.0    date 22-07-1998
c
c   copyright (c) 1997-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c **********************************************************************
c
c                       DESCRIPTION
c
c   Compute stress contribution to right-hand side for the Euler
c   equations for given pressure boundary condition, i.e.normal stress,
c   Wesbeek discretization.
c   Similar to routine isbwbs2, but only snn pieces kept here. If
c   boundary condition handling is changed here, also change isbwbs2.
c **********************************************************************
c
c                       KEYWORDS
c
c     2d
c     boundary_conditions
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, alphad, ibinfor(*),
     +        ibipoint(*), axisym
      double precision
     +   adir(1-nvirtual:ni+1+nvirtual,1-nvirtual:nj+1+nvirtual,2),
     +   adiri(1-nvirtual:ni+1+nvirtual,1-nvirtual:nj+1+nvirtual,2),
     +   rhside(ni+1,nj+1,2), bndcon(2,*), bcgeo(8,*),
     +   coor(1-nvirtual:1+ni+nvirtual,
     +        1-nvirtual:1+nj+nvirtual,1:2)

c     adir           i    Contravariant base vector in integration cell centre
c     adiri          i    Contravariant base vector in integration point
c     alphad         i    Direction index of integration.
c                         Hence if a V1-cell is used, alphad = 1 and so on
c     axisym         i    In axi-symmetric case axisym=1 else axisym=0
c     bcgeo          i    Array containing information about the geometrical
c                         quantities along the boundaries
c                         The contents of bcgeo depend on the dimension.
c                         In R2 the following definition is used:
c                         bcgeo(igeo,icount,k), igeo=1(1)8,icount=1(1)..,k=1,2
c                         with: icount defines the cell index, where the cells
c                                  are numbered along the boundaries
c                                  First all cells of boundary 1,
c                                  then of boundary 2 etc.
c                                  k = 1, corresponds to vertex of cell and
c                                  k = 2 to midside point of cell
c                                  igeo defines the type of quantity.
c                                  Possible values:
c                                  1: sqrt(g) g^11
c                                  2: sqrt(g) g^12
c                                  3: sqrt(g) g^22
c                                  4: sqrt(g) a^(1)_1
c                                  5: sqrt(g) a^(1)_2
c                                  6: sqrt(g) a^(2)_1
c                                  7: sqrt(g) a^(2)_2
c                                  8: sqrt(g)
c                         In R3 the following definition is used:
c                         bcgeo(igeo,i,j), igeo=1(1)12, i and j vary over all
c                                  i and j indices in the same way as ibinfor
c                                  which means that both vertices and mid points
c                                  are used
c                                  igeo defines the type of quantity.
c                                  Possible values:
c                                  1: sqrt(g) g^11  7: g_11 13: sqrt(g) a^(1)_1
c                                  2: sqrt(g) g^12  8: g_12 14: sqrt(g) a^(1)_2
c                                  3: sqrt(g) g^13  9: g_13 15: sqrt(g) a^(1)_3
c                                  4: sqrt(g) g^22 10: g_22 16: sqrt(g) a^(2)_1
c                                  5: sqrt(g) g^23 11: g_23 17: sqrt(g) a^(2)_2
c                                  6: sqrt(g) g^33 12: g_33 18: sqrt(g) a^(2)_3
c     bndcon         i    Array BNDCON is used to indicate the value of the
c                         boundary conditions for each point at the boundary of
c                         each single block for each unknown.
c                         For a description see the programmers guide
c     coor           i    Array containing the coordinates for one block
c                         In 2d coor(i,j,.) contains the x resp y-coordinate
c                         of point i,j
c                         in 3d: coor(i,j,k,.) corresponds to (i,j,k)
c     ibinfor        i    Third part of infbnd
c     ibipoint       i    ipoint(side)
c                                      position of start of ibinfor information
c                                      for SIDE in ibinfor
c     ni             i    Number of non-virtual cells in the i-direction
c                         (one block)
c     nj             i    Number of non-virtual cells in the j-direction
c                         (one block)
c     nvirtual       i    Number of rows of virtual cells
c     rhside        i/o   right-hand side vector
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer  btype,  ipibinf, tsign,
     +        ict1,  ip, jp, xp(2), ndir,
     +        tdir, nsign, iside, istwo(2,2),
     +        ilow, iupp, jlow, jupp, snnsnt, snnut
      double precision an1, an2,  gnn, fact, rmean, n1, n2
      parameter(  snnsnt = 7, snnut = 6)

c     an1            Component 1 of contravariant base vector in normal
c                    direction
c     an2            Component 2 of contravariant base vector in normal
c                    direction
c     btype          Type of boundary condition
c     fact           Multiplication factor
c     gnn            Contravariant metric tensor
c     ict1           Pointer in ibinfor with respect to boundary point
c     ilow           Lower index of subface
c     ip             Local counting variable i-direction
c     ipibinf        Starting address of array infor for present side
c                    minus one
c     iside          Sequence number of side
c     istwo          Translation table to store and retrieve the
c                    elements of a symmetric ndim-dimensional
c                    rank 2 tensor from a vector with length
c                    (ndim*(ndim+1))/2
c     iupp           Upper i-index of subface
c     jlow           Lower j-index for a subface
c     jp             Local counting variable j-direction
c     jupp           Upper j-index of subface
c     n1             Component 1 of normal vector
c     n2             Component 2 of normal vector
c     ndir           Direction of normal vector on this side
c     nsign          Inner product of outward normal in computational
c                    domain with the base vector perpendicular to the face
c     rmean          Mean value of r coordinate
c     snnsnt         Sigma_nn and sigma_nt prescribed
c     snnut          Sigma_nn and u_t prescribed
c     tdir           Direction of tangent vector on this side
c     tsign          Inner product of tangential vector defined counter
c                    clockwise with respect to the outward normal and the
c                    base vector along the side
c     xp             Auxiliary array to calculate boundary point number
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISORIENT2      Computes ndir, nsign, tdir, tsign for a 2-dimensional
c                    problem
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c   Consider a normal half cell at the lower boundary, i.e.
c       ntdir=(2,1)
c       ntsig=(-1,1)
c
c
c         ------------------------------------------
c         |              |            |            |
c         |              |            |            |
c         |              |            |            |
c         |              +-----+------+            |
c         |              |            |            |
c         |              |            |            |
c         |              |            |            |
c         ---------------+-----*------+-------------
c
c
c   The stress term for this cell has the following shape:
c
c    (a)       1          (1)  b |i+1/2,j     (2)          b |i,j+1/2
c   a      . { - sqrt(g) a    T  |         + a   sqrt(g)  T  |        }
c    (0,0)     2          b      |i-1/2,j     b              |i,j-1/2
c
c
c   The following boundary conditions have the following effects:
c
c
c   un given in boundary centroid:  matrix elements equal to 0
c
c   Sigma nn, Sigma nt given:       Replace all terms sigma nn and sigma nt
c                                   at the boundary by their given value
c                                   and multipli the contributions
c                                   at the half sides with factor 0.5 .
c
c      Formulae:
c
c
c     (2)          b |      ||  (2)        ||
c    a   sqrt(g)  T  |    = || a    sqrt(g)|| {sigma^nn n + sigma^nt t }
c     b              |i,j   ||             ||
c
c     with sigma^nn and sigma^nt given.
c
c
c   Sigma nn, ut given:             Replace all terms sigma nn
c                                   at the boundary by the given value
c                                   and multipli the contributions
c                                   at the half sides with factor 0.5 .
c                                   Use the folding subroutine to eliminate
c                                   virtual tangential velocities
c
c      Formulae:
c
c
c     (2)          b |      ||  (2)        ||                   b
c    a   sqrt(g)  T  |    = || a    sqrt(g)|| {sigma^nn n + n (T.t)t  }
c     b              |i,j   ||             ||                b - - -
c
c     with  sigma^nn given, t expressed in a
c                           -               (t)
c
c                               (n)
c     and  a     expressed in a
c           (t)
c
c       1                 (2)           1                 (1)
c     a     =   sqrt(g)  a            a     = - sqrt(g)  a
c      (1)                (2)          (2)                (2)
c
c       2                 (2)           2                 (1)
c     a     = - sqrt(g)  a            a     =   sqrt(g)  a
c      (1)                (2)          (2)                (1)
c
c
c
c    The comments refer to a cell at the lower boundary
c **********************************************************************
c
c                       DATA STATEMENTS
c
      data istwo / 1, 2,
     +             2, 3 /
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- Loop over all sides

      rmean = 1d0
      do iside = 1, 4

         ipibinf = ibipoint(iside)-1+nvirtual
         ilow = 1
         iupp = 1
         jlow = 1
         jupp = 1

c        --- Compute tdir and ndir

         call isorient2 ( iside, tdir, ndir, tsign, nsign )

         if ( iside.le.2 ) then

c        --- iside = 1, 2:  left- and right-hand side

            jlow = 1
            jupp = nj
            if ( iside.eq.2 ) then

c           --- iside = 2:  right-hand side

               ilow = ni+1
               iupp = ni+1

            end if

         else

c        --- iside = 3, 4:  lower- and upper side

            ilow = 1
            iupp = ni
            if ( iside.eq.4 ) then

c           --- iside = 4:  upper side

               jlow = nj+1
               jupp = nj+1

            end if

         end if
         if ( alphad.eq.ndir ) then

c        --- loop over cells

            do jp = jlow, jupp
               do ip = ilow, iupp

c              --- Get integration cell index

                  xp(1) = ip
                  xp(2) = jp
                  if ( axisym.eq.1 ) then
                     if ( iside.le.2 ) then
                        rmean = 0.5d0*(coor(ip,jp+1,1)+coor(ip,jp,1))
                     else
                        rmean = 0.5d0*(coor(ip+1,jp,1)+coor(ip,jp,1))
                     end if
                  end if

c                 --- Get boundary information

                  ict1 = ipibinf+2*xp(tdir)
                  btype = ibinfor(ict1)

                  if ( btype.eq.snnsnt ) then

c                 --- Both sigma^nn and sigma^nt given in integration point
c                     Get boundary condition information

                        an1 = bcgeo((2+2*ndir),ict1)/
     +                        bcgeo(8,ict1)
                        an2 = bcgeo((3+2*ndir),ict1)/
     +                        bcgeo(8,ict1)
                        gnn = bcgeo(istwo(ndir,ndir),ict1)/
     +                        bcgeo(8,ict1)
                        n1 = an1/sqrt(gnn)
                        n2 = an2/sqrt(gnn)

                        fact =  (adir(ip,jp,1)*n1 +
     +                            adir(ip,jp,2)*n2) *
     +                        sqrt(adiri(ip,jp,1)*adiri(ip,jp,1)+
     +                           adiri(ip,jp,2)*adiri(ip,jp,2))

                        rhside(ip,jp,alphad) =
     +                         rhside(ip,jp,alphad) +
     +                         fact * nsign * bndcon(1,ict1)*rmean

                  else if ( btype.eq.snnut ) then

c                 --- sigma^nn provided and ut provided

                        an1 = bcgeo((2+2*ndir),ict1)/
     +                        bcgeo(8,ict1)
                        an2 = bcgeo((3+2*ndir),ict1)/
     +                        bcgeo(8,ict1)
                        gnn = bcgeo(istwo(ndir,ndir),ict1)/
     +                        bcgeo(8,ict1)
                        n1 = an1/sqrt(gnn)
                        n2 = an2/sqrt(gnn)

                        fact = (adir(ip,jp,1)*n1 +
     +                            adir(ip,jp,2)*n2) *
     +                        sqrt(adiri(ip,jp,1)*adiri(ip,jp,1)+
     +                           adiri(ip,jp,2)*adiri(ip,jp,2))

                        rhside(ip,jp,alphad) =
     +                         rhside(ip,jp,alphad) +
     +                         fact * nsign * bndcon(1,ict1)*rmean

                  end if

               end do
            end do
         end if
      end do

      end
