      subroutine isdisdt3 ( alphad, ni, nj, nk, nvirtual, matrix, dtsum,
     +                      mct, iarea, dcell, ichoisdis )
c ======================================================================
c
c        programmer     Marcel Zijlema
c        version 1.0    date   01-05-1996
c
c
c   copyright (c) 1995-1996  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Distribute diffusion term for tranport equation in R3
c     Both standard distribution as well as monotone distributions are
c     possible
c **********************************************************************
c
c                       KEYWORDS
c
c     discretization
c     transport_matrix
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nk, nvirtual, iarea(2,3), dcell(3), alphad,
     +        ichoisdis, mct(-2:2,-2:2,-2:2)
      double precision matrix(ni+1,nj+1,nk+1,*),
     +                 dtsum(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,
     +                       1-nvirtual:nk+1+nvirtual,3)

c     alphad    i    Direction index of integration.
c                    Hence if a V1-cell is used, alphad = 1 and so on
c     dcell     i    reserved
c     dtsum     i    Flux derivative contributions to the matrix
c     iarea     i    Area of arrays
c     ichoisdis i    Choice parameter with respect to the convection scheme
c                    for discretization of convective term of momentum and
c                    transport equations.
c                    Meaning of ichoisdis:
c                      ichoisdis =  0: Second order central scheme
c                      ichoisdis =  1: First order upwind scheme
c                      ichoisdis =  2: Hybrid central/upwind scheme
c                      ichoisdis =  3: Higher order upwind scheme (kappa-scheme)
c                      ichoisdis =  4: Sweby Phi-limiter
c                      ichoisdis =  5: R-kappa limiter
c                      ichoisdis =  6: MR-kappa limiter
c                      ichoisdis =  7: Symmetric Ratio limiter
c                      ichoisdis =  8: PL-kappa limiter
c                      ichoisdis =  9: MPL1-kappa limiter
c                      ichoisdis = 10: MPL2-kappa limiter
c                      ichoisdis = 11: Symmetric PL-kappa limiter
c     matrix   i/o   reserved
c     mct       i    Integer matrix information
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ishift, jshift, kshift, ip, jp, kp, betad, gammad, minusg,
     +        dx(3,3), ofstnr(10)
      double precision faca, facb, facc

c     betad     Direction index, usually the direction perpendicular to the
c               integration direction of the boundary integral.
c               For example betad in local point (1,0) is 1 and in (0,1): 2
c     dx        Point location in cell relative to local
c               coordinates with (0,0) and (2,2) or
c               (0,0,0) and (2,2,2) as opposite cell corners
c     faca      A factor used in the linear extrapolation formulae
c     facb      A factor used in the linear extrapolation formulae
c     facc      A factor used in the linear extrapolation formulae
c     gammad    Direction index
c     ip        Local counting variable i-direction
c     ishift    Shift in i-direction
c     jp        Local counting variable j-direction
c     jshift    Shift in j-direction
c     kp        Local counting variable k-direction
c     kshift    Shift in k-direction
c     minusg    Indicates if a minus sign has been found (-1) or not (1)
c     ofstnr    Offset diagonal number of matrix element that
c               corresponds to the current point of the interpolation
c               or extrapolation formula
c **********************************************************************
c
c                       I/O
c
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c    Consider a transport cell with surrounding cells in a 2D plane:
c
c            -------------------------------------
c            |           |           |           |
c            |     +     |     +     |     +     |
c            |           |           |           |
c            ------------------|------------------
c            |           |           |           |
c            |     +     -     +     -     +     |
c            |           |           |           |
c            ------------------|------------------
c            |           |           |           |
c            |     +     |     +     |     +     |
c            |           |           |           |
c            -------------------------------------
c
c
c     If the integration point is in the 1-direction (locdx(1)=+/-1)
c     then the contribution in the integration point must be added to
c     the correct matrix elements multiplied by a factor.
c
c     Since dtsum contains the partial derivatives the factors are
c     given by the finite difference formula.
c     For example in (1,0,0) we have:
c
c     dT/dksi_1 = T(2,0,0) - T(0,0,0)
c     dT/dksi_2 = (T(2,2,0) - T(2,0,0) + T(0,2,0) - T(0,0,0) ) / 4
c
c     If ichoisdis=1 we use the scheme proposed by Demirdzic (1982)
c     This scheme, is based on 2-point
c     rather than 4-point interpolation. For example, assuming that we
c     want to discretize the following term
c
c      12 dT                         12
c     K   --  at (i+1/2,j,k), where K   is a contravariant diffusion tensor
c         dx2
c                       12
c     and suppose that K  (i+1/2,j,k) > 0, then the following approximation is
c     taken
c
c     dT                1
c     -- |            = - (T        + T           - T          - T         )
c     dx2 (i+1/2,j,k)   2   (i,j,k)    (i+1,j+1,k)   (i,j-1,k)    (i+1,j,k)
c
c
c     The interpolation scheme is described in:
c
c     Finite volume discretization of the k-eps turbulence model in
c     general coordinates,
c     Marcel Zijlema,
c     TUD Report 93-90, 1993.
c
c     If ichoisdis=2 we use the scheme proposed by Zijlema (1993)
c     This scheme, is rather
c     one-sided than central difference interpolation. For example,
c     assuming that we want to discretize the following term
c
c      12 dT                         12
c     K   --  at (i+1/2,j,k), where K   is a contravariant diffusion tensor
c         dx2
c                       12
c     and suppose that K  (i+1/2,j,k) > 0, then the following approximation is
c     taken
c
c     dT                1
c     -- |            = - (T            + T         )  - T
c     dx2 (i+1/2,j,k)   2   (i+1,j+1,k)    (i,j+1,k)      (i,j,k)
c
c
c     The interpolation scheme is described in:
c
c     Finite volume discretization of the k-eps turbulence model in
c     general coordinates,
c     Marcel Zijlema,
c     TUD Report 93-90, 1993.
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
c     --- Shift from integration cell to dtsum; i.e. to cell
c         containing the integration point
c         (Rule: for each grid cell left and lower side are included,
c          top and right hand side are in the next cell)

      ishift = dcell(1)
      jshift = dcell(2)
      kshift = dcell(3)

c     --- Compute the two connections for the standard derivatives

      dx(1,1) = 0
      dx(2,1) = 0
      dx(3,1) = 0
      dx(alphad,1) = 1
      betad  = mod(alphad  ,3) + 1
      gammad = mod(alphad+1,3) + 1
      ofstnr(1) = mct(ishift,jshift,kshift)
      ofstnr(2) = mct(ishift-dx(1,1),jshift-dx(2,1),kshift-dx(3,1))

c     --- Make a choice between central, monotonous and half monotonous scheme

      if ( ichoisdis.eq.0 ) then

c     --- central scheme
c         Compute the four connections for the cross derivatives

         if ( alphad.eq.1 ) then
            ofstnr(3)  = mct(ishift  , 1, 0)
            ofstnr(4)  = mct(ishift-1, 1, 0)
            ofstnr(5)  = mct(ishift  ,-1, 0)
            ofstnr(6)  = mct(ishift-1,-1, 0)
            ofstnr(7)  = mct(ishift  , 0, 1)
            ofstnr(8)  = mct(ishift-1, 0, 1)
            ofstnr(9)  = mct(ishift  , 0,-1)
            ofstnr(10) = mct(ishift-1, 0,-1)
         else if ( alphad.eq.2 ) then
            ofstnr(3)  = mct( 0,jshift  , 1)
            ofstnr(4)  = mct( 0,jshift-1, 1)
            ofstnr(5)  = mct( 0,jshift  ,-1)
            ofstnr(6)  = mct( 0,jshift-1,-1)
            ofstnr(7)  = mct( 1,jshift  , 0)
            ofstnr(8)  = mct( 1,jshift-1, 0)
            ofstnr(9)  = mct(-1,jshift  , 0)
            ofstnr(10) = mct(-1,jshift-1, 0)
         else
            ofstnr(3)  = mct( 1, 0,kshift  )
            ofstnr(4)  = mct( 1, 0,kshift-1)
            ofstnr(5)  = mct(-1, 0,kshift  )
            ofstnr(6)  = mct(-1, 0,kshift-1)
            ofstnr(7)  = mct( 0, 1,kshift  )
            ofstnr(8)  = mct( 0, 1,kshift-1)
            ofstnr(9)  = mct( 0,-1,kshift  )
            ofstnr(10) = mct( 0,-1,kshift-1)
         end if

         do kp = iarea(1,3),iarea(2,3)
           do jp = iarea(1,2),iarea(2,2)
             do ip = iarea(1,1),iarea(2,1)
               faca = dtsum(ip+ishift,jp+jshift,kp+kshift,alphad)
               facb = dtsum(ip+ishift,jp+jshift,kp+kshift,betad)*
     +                0.25d0
               facc = dtsum(ip+ishift,jp+jshift,kp+kshift,gammad)*
     +                0.25d0
               matrix(ip,jp,kp,ofstnr(1)) = matrix(ip,jp,kp,ofstnr(1)) +
     +                                      faca
               matrix(ip,jp,kp,ofstnr(2)) = matrix(ip,jp,kp,ofstnr(2)) -
     +                                      faca
               matrix(ip,jp,kp,ofstnr(3)) = matrix(ip,jp,kp,ofstnr(3)) +
     +                                      facb
               matrix(ip,jp,kp,ofstnr(4)) = matrix(ip,jp,kp,ofstnr(4)) +
     +                                      facb
               matrix(ip,jp,kp,ofstnr(5)) = matrix(ip,jp,kp,ofstnr(5)) -
     +                                      facb
               matrix(ip,jp,kp,ofstnr(6)) = matrix(ip,jp,kp,ofstnr(6)) -
     +                                      facb
               matrix(ip,jp,kp,ofstnr(7)) = matrix(ip,jp,kp,ofstnr(7)) +
     +                                      facc
               matrix(ip,jp,kp,ofstnr(8)) = matrix(ip,jp,kp,ofstnr(8)) +
     +                                      facc
               matrix(ip,jp,kp,ofstnr(9)) = matrix(ip,jp,kp,ofstnr(9)) -
     +                                      facc
               matrix(ip,jp,kp,ofstnr(10)) =matrix(ip,jp,kp,ofstnr(10))-
     +                                      facc
             end do
           end do
         end do

      else if ( ichoisdis.eq.2 ) then

c     --- half monotonous scheme
c         NOT YET IMPLEMENTED

      else

c     --- monotonous scheme
c         NOT YET IMPLEMENTED

      end if

      end
