      subroutine isdrhodh ( ndim, ni, nj, nk, nvirtual, solut, eddy,
     +                      rinput, iinput, ndegfd, drhodh )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.0    date 30-06-1999
c
c   copyright (c) 1999-1999  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Compute drho/dh from the equation of state
c
c **********************************************************************
c
c                       KEYWORDS
c
c    2d
c    matrix
c    pressure_equation
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ndim, ni, nj, nk, nvirtual, iinput(*), ndegfd
      double precision drhodh(1-nvirtual:ni+1+nvirtual,
     +                        1-nvirtual:nj+1+nvirtual), eddy(*),
     +                 solut(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,ndegfd,*),
     +                 rinput(*)

c     drhodh         o    Contains drho/dh in all centres of the cells
c     eddy           i    array EDDY contains the values of the
c                         eddy viscosity of that block in the centroids
c                         of the elements. The storage of such
c                         an array EDDY is identical to that of all other
c                         coefficients.
c     iinput         i    Contains the integer user input for one block.
c                         For a description see the programmers guide
c     ndegfd         i    Number of degrees of freedom
c     ndim           i    Dimension of space (2 or 3)
c     ni             i    Number of non-virtual cells in the i-direction
c                         (one block)
c     nj             i    Number of non-virtual cells in the j-direction
c                         (one block)
c     nk             i    Number of non-virtual cells in the k-direction
c                         (one block)
c     nvirtual       i    Number of rows of virtual cells
c     rinput         i    The real user input is stored in the array rinput
c                         for each block
c                         For a description see the programmers guide
c     solut          i    Solution array
c                         For a description see the programmers guide
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, j, iprincom, mscal, meqstate, ipiincom
      double precision gamcm, mach2, cons, cons1,
     +                 cons2, pv, fact

c     cons           Constant used in equation of state for compressible
c                    computations
c     cons1          Constant used in equation of state for compressible
c                    computations
c     cons2          Constant used in equation of state for compressible
c                    computations
c     fact           Multiplication factor
c     gamcm          Gas constant gamma (for compressible comput.)
c     i              General loop variable
c     ipiincom       Starting address of array IINCOM in array IINPUT
c     iprincom       Starting address of array RINCOM in array RINPUT
c     j              General loop variable
c     mach2          Square of the mach number (for compressible comput.)
c     meqstate       Defines the type of equation of state
c                    See the programmers Guide array IINCOM for the
c                    possible values
c     mscal          Indicates use of non-dimensionalisation for
c                    compr. flow:
c                    0:   Equations are non-dimensionalised
c                    1:   Equations are in dimensional formulation
c     pv             Ratio of outlet and inlet pressure used for
c                    compressible flow in the equation of state
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     USDRHODH       User subroutine to compute drho/dh
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c    drho/dh follows immediately from the equation of state
c    For a perfect gas we have:
c
c    rho = gamma/(gamma-1) p/h, hence drho/dh = -gamma p/((gamma-1) h^2)
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c

      ipiincom  = iinput(9)
      meqstate  = iinput(ipiincom+9-1)

      mscal    = iinput(ipiincom+4)
      iprincom = iinput(34)-1

      if ( meqstate.eq.0 ) then

c     --- meqstate = 0, perfect gas: rho = gamma/(gamma-1) / p

         gamcm = rinput(iprincom+1)
         mach2 = rinput(iprincom+2)**2d0
         pv    = rinput(iprincom+5)
         fact=gamcm/(gamcm-1d0)
         cons  = 1d0 + ((gamcm - 1d0)*mach2)/2d0
         cons1 = (gamcm * mach2) / cons
         cons2 = pv * (cons**(-fact) - 1d0) + 1d0
         if ( mscal.eq.0 ) then

c        --- mscal = 0, scaled (dimensionless form)

            do j = 1-nvirtual, nj+1+nvirtual
               do i = 1-nvirtual, ni+1+nvirtual
                  drhodh(i,j) = (cons1*solut(i,j,3,1) + cons2) /
     +                          solut(i,j,4,1)**2

               end do
            end do

         else

c        --- mscal = 1, not scaled (dimensionfull form)

            do j = 1-nvirtual, nj+1+nvirtual
               do i = 1-nvirtual, ni+1+nvirtual
                  drhodh(i,j) = fact*solut(i,j,3,1) /
     +                          solut(i,j,4,1)**2

               end do
            end do

         end if

      else

c     --- meqstate = 1, user defined equation of state
c         Call subroutine to compute drho/dp

         call usdrhodh ( eddy, solut, ni, nj, nk, nvirtual, ndim,
     +                   ndegfd, drhodh )
      end if

      end
