      subroutine ismakmp2 ( intn, alphad, locdx, pnttyp, dcell,
     +                      nintpol, dij )
c ======================================================================
c
c        programmer     Guus Segal
c        version 2.0    date 16-10-1995 (Complete new subroutine)
c        version 1.0    date 14-03-1995 (Extracted from subroutine ismpts)
c
c   copyright (c) 1995-1995  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c **********************************************************************
c
c                       DESCRIPTION
c
c     Compute some quantities with respect to interpolation, position and so
c     on for one integration point and one direction. (2D)
c **********************************************************************
c
c                       KEYWORDS
c
c    matrix
c    right_hand_side
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer intn, alphad, locdx(2), pnttyp, dcell(2), nintpol(2),
     +        dij(2,4,2)

c     alphad    i    Direction index of integration.
c                    Hence if a V1-cell is used, alphad = 1 and so on
c     dcell     o    reserved
c     dij       o    Array containing the distances of the interpolation
c                    points to the cell centra in global co-ordinates
c                    dij(alpha,i,j) denotes the distance of the i-th
c                    interpolation point, component j and with respect to
c                    the V-alpha unknown (distribution)
c     intn      i    Integration point number
c     locdx     o    Position of the integration point with respect to
c                    the centre of the integration cell
c                    The cell sizes are assumed to be 2 times 2, hence the
c                    distance of a midside point to a cell centre is always 1
c     nintpol   o    Number of interpolation points in the distribution
c     pnttyp    o    reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer betad

c     betad     Direction index, usually the direction perpendicular to the
c               integration direction of the boundary integral.
c               For example betad in local point (1,0) is 1 and in (0,1): 2
c **********************************************************************
c
c                       I/O
c
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c        Create first 2D list of local coordinates of the
c        integration points w.r.t. the integration cell centre,
c        for the momentum matrix integration cells. These local
c        coordinates are stored in array locdx, which is filled
c        for one celltype.
c        For each integration point its pnttype is stored in
c        array pnttyp.
c        pnttyp       Point where a value or finite difference is
c                     to be calculated within the cell
c
c                       ksi^2
c                        ^     +----|----+
c                        |     |         |
c                              |         |
c                             --    *    --
c                            3 |   4     |
c                              |         |
c                              +----|----+   -> ksi^1
c                             1     2
c
c        In 2D the number of integration points for the internal
c                             1
c        cells is 5, for the V  cell point 1 is integration cell centre,
c        points 2 and 3 are the midpoints of the left and rigth side,
c        midpoints of the left and rigth side, points 4 and 5 are the
c        midpoints of the lower and upper side of the integration cell
c
c          1
c         V  integration cell with integration points in 2D
c
c                 5           in the 1 direction     in the 2 direction
c           -----*-----
c                |              locdx(1,1) = 0        locdx(2,1) = 0
c                |              locdx(1,2) = -1       locdx(2,2) = 0
c          2*   1-    3*        locdx(1,3) = 1        locdx(2,3) = 0
c    2           |              locdx(1,4) = 0        locdx(2,4) = -1
c      ^         |4             locdx(1,5) = 0        locdx(2,5) = 1
c      |    -----*-----
c      |                        pnttyp(1) = 3
c       -----> 1                pnttyp(2) = 4
c                               pnttyp(3) = 4
c                               pnttyp(4) = 1
c                               pnttyp(5) = 1
c
c                               dcell(1,1) = 0        dcell(2,1) = 0
c                               dcell(1,2) = -1       dcell(2,2) = 0
c                               dcell(1,3) = 0        dcell(2,3) = 0
c                               dcell(1,4) = 0        dcell(2,4) = 0
c                               dcell(1,5) = 0        dcell(2,5) = 1
c                  2
c        For the V  cell
c        point 1 is integration cell centre, points 4 and 5 are the
c        midpoints of the left and rigth side, points 2 and 3 are the
c        midpoints of the lower and upper side of the integration cell,
c        locdx is now filled as follows.
c
c          2
c         V  integration cell with integration points in 2D
c
c                 3          in the 1 direction     in the 2 direction
c                *
c                |              locdx(1,1) = 0       locdx(2,1) = 0
c                |              locdx(1,2) = 0       locdx(2,2) = -1
c          4*----1----5*        locdx(1,3) = 0       locdx(2,3) = 1
c    2           |              locdx(1,4) = -1      locdx(2,4) = 0
c      ^         |2             locdx(1,5) = 1       locdx(2,5) = 0
c      |         *
c      |                        pnttyp(1) = 2
c       -----> 1                pnttyp(2) = 4
c                               pnttyp(3) = 4
c                               pnttyp(4) = 1
c                               pnttyp(5) = 1
c
c                               dcell(1,1) = 0        dcell(2,1) = 0
c                               dcell(1,2) = 0        dcell(2,2) = -1
c                               dcell(1,3) = 0        dcell(2,3) = 0
c                               dcell(1,4) = 0        dcell(2,4) = 0
c                               dcell(1,5) = 1        dcell(2,5) = 0
c
c
c     end if
c
c    With respect to the momentum equations we have the following conventions:
c
c    alphad is equal to the V alphad unknown in the centre
c    betad is equal to alphad+1 (cyclic)
c    gammad is equal to alphad+2 (cyclic)
c
c    With respect to the interpolation we have the following conventions:
c
c      Consider a V1 cell with 5 integration points
c
c      The coefficient of a velocity coefficient in the equation
c      at the integration point is given.
c      Since the velocity does not have to be present at this integration point
c      it is necessary to distribute the coefficient over surrounding
c      neighbours.
c      For example V2 in a V1 point (pnttyp=3) is created by
c      the linear combination: (V2(i,j)+V2(i,j+1)+V2(i-1,j)+V2(i-1,j+1))/4
c
c          *----------------*-----------------*
c          |                |                 |
c          |                |                 |
c          |                |                 |
c          *----------------*-----------------*
c          |        |       |        |        |
c          |                -i,j              |
c          |        |       |        |        |
c          *----------------*-----------------*
c
c
c      This subroutine gives the number of distribution points as well
c      as the positions in the row of the matrix where the coefficients
c      must be put as function of the direction alphad, the position of the
c      integration point in local co-ordinates locdx, for V1 and V2
c      velocity components
c
c      The positions in the row of the matrix will be called connections
c      They are computed using the array mct.
c      Array mct depends on alphad, the local direction, the point type
c      and also of the position of the point relative to the cell centre.
c      With respect to the V alphad coefficient the position is equal to
c      the local co-ordinate divided by two.
c      With respect to the other component we need the following table:
c
c       Local co-ordinate      V1                V2
c
c        (-1,-1)             (-1, 0)           ( 0,-1)
c        ( 1,-1)             ( 0, 0)           ( 1,-1)
c        (-1, 1)             (-1, 1)           ( 0, 0)
c        ( 1, 1)             ( 0, 1)           ( 1, 0)
c
c      The distribution is given by the following table
c
c      alphad  locdx  pnttyp betad  nintpol  local co-ordinates:
c
c          1   (0,0)   3     1       1       (0,0)
c                            2       4       (+/- 1, +/- 1)
c            (+/-1,0)  4     1       2       (0,0), locdx
c                            2       2       (locdx(1), +/- 1)
c            (0,+/-1)  1     1       2       (0,0), locdx
c                            2       2       (+/- 1,locdx(2))
c
c          2   (0,0)   2     2       1       (0,0)
c                            1       4       (+/- 1, +/- 1)
c            (+/-1,0)  1     2       2       (0,0), locdx
c                            1       2       (locdx(1), +/- 1)
c            (0,+/-1)  4     2       2       (0,0), locdx
c                            1       2       (+/- 1,locdx(2))
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      betad  = 3-alphad

      go to ( 100, 200, 300, 400, 500 ), intn

c     --- Integration cell centre          integration point 1:(0,0)

100      locdx(1) =  0
         locdx(2) =  0
         pnttyp   =  4-alphad
         dcell(1) =  0
         dcell(2) =  0
         nintpol(alphad) = 1
         dij(alphad,1,1) = 0
         dij(alphad,1,2) = 0
         nintpol(betad) = 4
         dij(betad,1,alphad) = 0
         dij(betad,1,betad) = 1
         dij(betad,2,alphad) = 0
         dij(betad,2,betad) = 0
         dij(betad,3,alphad) = -1
         dij(betad,3,betad) = 1
         dij(betad,4,alphad) = -1
         dij(betad,4,betad) = 0
         go to 1000

c        --- integration point 2: V1:(-1,0),   V2:(0,-1)

200      locdx(alphad) = -1
         locdx(betad)  =  0
         pnttyp        =  4
         dcell(1)      =  locdx(1)
         dcell(2)      =  locdx(2)
         nintpol(alphad) = 2
         dij(alphad,1,1) = 0
         dij(alphad,1,2) = 0
         dij(alphad,2,1) = locdx(1)
         dij(alphad,2,2) = locdx(2)
         nintpol(betad) = 2
         dij(betad,1,alphad) = -1
         dij(betad,1,betad) = 0
         dij(betad,2,alphad) = -1
         dij(betad,2,betad) = 1
         go to 1000

c        --- integration point 3: V1:(1,0),   V2:(0,1)

300      locdx(alphad) =  1
         locdx(betad)  =  0
         pnttyp        =  4
         dcell(1)      =  0
         dcell(2)      =  0
         nintpol(alphad) = 2
         dij(alphad,1,1) = 0
         dij(alphad,1,2) = 0
         dij(alphad,2,1) = locdx(1)
         dij(alphad,2,2) = locdx(2)
         nintpol(betad) = 2
         dij(betad,1,alphad) = 0
         dij(betad,1,betad) = 0
         dij(betad,2,alphad) = 0
         dij(betad,2,betad) = 1
         go to 1000

c        --- integration point 4: V1:(0,-1),   V2:(-1,0)

400      locdx(alphad) =  0
         locdx(betad)  = -1
         pnttyp        =  1
         dcell(1)      =  0
         dcell(2)      =  0
         nintpol(alphad) = 2
         dij(alphad,1,1) = 0
         dij(alphad,1,2) = 0
         dij(alphad,2,1) = locdx(1)
         dij(alphad,2,2) = locdx(2)
         nintpol(betad) = 2
         dij(betad,1,alphad) = -1
         dij(betad,1,betad) = 0
         dij(betad,2,alphad) = 0
         dij(betad,2,betad) = 0
         go to 1000

c        --- integration point 5: V1:(0,1),   V2:(1,0)

500      locdx(alphad) =  0
         locdx(betad)  =  1
         pnttyp        =  1
         dcell(1)      =  locdx(1)
         dcell(2)      =  locdx(2)
         nintpol(alphad) = 2
         dij(alphad,1,1) = 0
         dij(alphad,1,2) = 0
         dij(alphad,2,1) = locdx(1)
         dij(alphad,2,2) = locdx(2)
         nintpol(betad) = 2
         dij(betad,1,alphad) = -1
         dij(betad,1,betad) = 1
         dij(betad,2,alphad) = 0
         dij(betad,2,betad) = 1

1000  continue

      end
