      subroutine ismakmp3 ( vartyp, ptsused, vtlist, lptlist, codelist )
c ======================================================================
c
c        programmer     Kees Kassels
c        version 1.0    date 01-05-1995 (Extracted from subroutine ismpts
c                                      as ismakmp2 )
c
c   copyright (c) 1995-1995  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c **********************************************************************
c
c                       DESCRIPTION
c
c     Give a list of integration points for a given momentum integration
c     cell. (3D)
c     Note: for different point lists, lptlist
c     AND vtlist must be changed. Moreover, some quantities
c     can only be calculated in the points given in codelist,
c     not in arbitrary points
c **********************************************************************
c
c                       KEYWORDS
c
c    matrix
c    right_hand_side
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer vartyp, ptsused, codelist(*), vtlist(3,*), lptlist(3,*)

c     codelist  o    List of point types (pnttyp) for the integration points
c     lptlist   o    Contains the position of the integration points relative to
c                    the integration cell centre measured in local co-ordinates
c     ptsused   o    Number of points in use
c     vartyp    i    Variable to be interpolated/extrapolated
c                    vartyp: 1 for V1, 2 for V2 and 3 for pressure,
c                    0 for vertex (for ndim=2)
c                    or
c                    vartyp: 1 for V1, 2 for V2 3, for V3 and
c                    4 for pressure, 0 for vertex  (for ndim=3)
c     vtlist    o    Contains for each integration point the shift of the cell
c                    index of this integration point with respect to the cell
c                    containing the centre of the integration cell.
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer alphad, betad, gammad, i, intn

c     alphad    Direction index of integration.
c               Hence if a V1-cell is used, alphad = 1 and so on
c     betad     Direction index, usually the direction perpendicular to the
c               integration direction of the boundary integral.
c               For example betad in local point (1,0) is 1 and in (0,1): 2
c     gammad    Direction index
c     i         General loop variable
c     intn      Integration point number
c **********************************************************************
c
c                       I/O
c
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c     Note for multiblock read multiblock, periodical or anti-periodical
c
c     Verify parameters
c     Initialize output arrays
c
c     ---Create first 2D list of local coordinates of the
c        integration points w.r.t. the integration cell centre,
c        for the momentum matrix integration cells. These local
c        coordinates are stored in array lptlist, which is filled
c        for one celltype.
c        For each integration point its pnttype is stored in
c        array codelist.
c        pnttyp       Point where a value or finite difference is
c                     to be calculated within the cell
c
c                       ksi^2
c                        ^     +----|----+
c                        |     |         |
c                              |         |
c                             --    *    --
c                            3 |   4     |
c                              |         |
c                              +----|----+   -> ksi^1
c                             1     2
c
c        In 2D the number of integration points for the internal
c                             1
c        cells is 5, for the V  cell point 1 is integration cell centre,
c        points 2 and 3 are the midpoints of the left and rigth side,
c        midpoints of the left and rigth side, points 4 and 5 are the
c        midpoints of the lower and upper side of the integration cell
c
c          1
c         V  integration cell with integration points in 2D
c
c                 5           in the 1 direction     in the 2 direction
c           -----*-----
c                |              lptlist(1,1) = 0        lptlist(2,1) = 0
c                |              lptlist(1,2) = -1       lptlist(2,2) = 0
c          2*   1-    3*        lptlist(1,3) = 1        lptlist(2,3) = 0
c    2           |              lptlist(1,4) = 0        lptlist(2,4) = -1
c      ^         |4             lptlist(1,5) = 0        lptlist(2,5) = 1
c      |    -----*-----
c      |                        codelist(1) = 3
c       -----> 1                codelist(2) = 4
c                               codelist(3) = 4
c                               codelist(4) = 1
c                               codelist(5) = 1
c
c                               vtlist(1,1) = 0        vtlist(2,1) = 0
c                               vtlist(1,2) = -1       vtlist(2,2) = 0
c                               vtlist(1,3) = 0        vtlist(2,3) = 0
c                               vtlist(1,4) = 0        vtlist(2,4) = 0
c                               vtlist(1,5) = 0        vtlist(2,5) = 1
c                  2
c        For the V  cell
c        point 1 is integration cell centre, points 4 and 5 are the
c        midpoints of the left and rigth side, points 2 and 3 are the
c        midpoints of the lower and upper side of the integration cell,
c        lptlist is now filled as follows.
c
c          2
c         V  integration cell with integration points in 2D
c
c                 3          in the 1 direction     in the 2 direction
c                *
c                |              lptlist(1,1) = 0       lptlist(2,1) = 0
c                |              lptlist(1,2) = 0       lptlist(2,2) = -1
c          4*----1----5*        lptlist(1,3) = 0       lptlist(2,3) = 1
c    2           |              lptlist(1,4) = -1      lptlist(2,4) = 0
c      ^         |2             lptlist(1,5) = 1       lptlist(2,5) = 0
c      |         *
c      |                        codelist(1) = 2
c       -----> 1                codelist(2) = 4
c                               codelist(3) = 4
c                               codelist(4) = 1
c                               codelist(5) = 1
c
c                               vtlist(1,1) = 0        vtlist(2,1) = 0
c                               vtlist(1,2) = 0        vtlist(2,2) = -1
c                               vtlist(1,3) = 0        vtlist(2,3) = 0
c                               vtlist(1,4) = 0        vtlist(2,4) = 0
c                               vtlist(1,5) = 1        vtlist(2,5) = 0
c
c
c     end if
c
c    With respect to the momentum equations we have the following conventions:
c
c    alphad is equal to the V alphad unknown in the centre
c    betad is equal to alphad+1 (cyclic)
c    gammad is equal to alphad+2 (cyclic)
c
c    Comments are written for a V1-cell (internal) or a cell at a lower
c    boundary (in R^2: alphad=2, betad=1; in R^3: alphad=3, betad=1, gammad=2)
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
c     --- initialize output arrays

      do 110 i = 1, 7
         do 100 alphad = 1, 3
            vtlist(alphad,i) = 0
100      continue
110   continue

      if ( vartyp.eq.1 ) then
         alphad  = 1
         betad   = 2
         gammad  = 3
      else if ( vartyp.eq.2 ) then
         alphad  = 2
         betad   = 3
         gammad  = 1
      else
         alphad  = 3
         betad   = 1
         gammad  = 2
      end if

c     --- integration cell centre: (0,0,0)
c         codelist = (7,6,4)

            intn = 1
            lptlist(alphad,intn) =  0
            lptlist(betad,intn)  =  0
            lptlist(gammad,intn) =  0
            codelist(intn)       =  8-2**(alphad-1)

c           --- integration point (-1,0,0)
c               codelist = (8,8,8)

            intn = 2
            lptlist(alphad,intn) = -1
            lptlist(betad,intn)  =  0
            lptlist(gammad,intn) =  0
            codelist(intn)       =  8
            vtlist(alphad,intn)  = -1

c           --- integration point (1,0,0)

            intn = 3
            lptlist(alphad,intn) =  1
            lptlist(betad,intn)  =  0
            lptlist(gammad,intn) =  0
            codelist(intn)       =  8

c           --- integration point (0,-1,0)
c               codelist = (5,2,3)

            intn = 4
            lptlist(alphad,intn) =  0
            lptlist(betad,intn)  = -1
            lptlist(gammad,intn) =  0
            codelist(intn)       =  2**(gammad-1)+1

c           --- integration point (0,1,0)
c               codelist = (5,2,3)

            intn = 5
            lptlist(alphad,intn) =  0
            lptlist(betad,intn)  =  1
            lptlist(gammad,intn) =  0
            codelist(intn)       =  2**(gammad-1)+1
            vtlist(betad,intn)   =  1

c           --- integration point (0,0,-1)
c               codelist = (3,5,2)

            intn = 6
            lptlist(alphad,intn) =  0
            lptlist(betad,intn)  =  0
            lptlist(gammad,intn) = -1
            codelist(intn)       =  2**(betad-1)+1

c           --- integration point (0,0,1)
c               codelist = (3,5,2)

            intn = 7
            lptlist(alphad,intn) =  0
            lptlist(betad,intn)  =  0
            lptlist(gammad,intn) =  1
            codelist(intn)       =  2**(betad-1)+1
            vtlist(gammad,intn)  =  1

      ptsused = intn

      end
