      subroutine ismomccv2 ( ni, nj, nvirtual, densrho, chsym,
     +                       veloc, vsum, alphad, intp, iindsc, mcom)
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.2    date 27-08-1999  Duncan: put invjac in distribution
c                                                routines
c        version 1.1    date 23-03-1999  Duncan: extended for compressible
c                                        flow with Newton linearisation
c        version 1.0    date 18-02-1998
c
c   copyright (c) 1998-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     determines the coefficients of the velocity components in
c     a given integration point within a flux cell for the convection term in R2
c **********************************************************************
c
c                       KEYWORDS
c
c    convection
c    matrix
c    right_hand_side
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, alphad, intp, iindsc(*), mcom
      double precision densrho(1-nvirtual:ni+1+nvirtual,
     +                         1-nvirtual:nj+1+nvirtual),
     +                 veloc(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,2)
      double precision vsum(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,2),
     +                 chsym(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,2,3)

c     alphad    i    Direction index of integration.
c                    Hence if a V1-cell is used, alphad = 1 and so on
c     chsym     i    Christoffel symbols of the second kind in the
c                    point given by pnttyp
c                    (symmetric contravariant part stored as vector
c                    with index encoding given by istwo)
c     densrho   i    Array containing the density rho. The size and structure is
c                    the same as for the scalar unknowns
c     iindsc    i    reserved
c     intp      i    Interpolated values of variable
c     mcom      i    If zero the flow is incompressible else (1) compressible
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     veloc     i    Array for the old velocities (used in convection
c                    term calculation)
c     vsum     i/o   Flux contributions to the matrix
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ip, jp, i, betad, mlin, istwo(2,2),
     +        ilow, jlow, iupp, jupp, iconserv, iupwind
      double precision  facs

c     betad     Direction index, usually the direction perpendicular to the
c               integration direction of the boundary integral.
c               For example betad in local point (1,0) is 1 and in (0,1): 2
c     facs      Multiplication factor
c     i         General loop variable
c     iconserv  If 1 the conservative scheme is used, otherwise (0) the
c               non-conservative one
c     ilow      Lower index of subface
c     ip        Local counting variable i-direction
c     istwo     Translation table to store and retrieve the
c               elements of a symmetric ndim-dimensional
c               rank 2 tensor from a vector with length
c               (ndim*(ndim+1))/2
c     iupp      Upper i-index of subface
c     iupwind   Type of upwinding
c     jlow      Lower j-index for a subface
c     jp        Local counting variable j-direction
c     jupp      Upper j-index of subface
c     mlin      Type of linearization (1=newton, 2=picard)
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c       Relevant formula
c
c
c    The convective terms in case of the classical discretization are given by:
c
c
c    rho   a  1 |i+1/2,j     rho      a  2 |i,j+1/2
c   ----- V  V  |         + -----    V  V  |          +
c   sqrt(g)     |i-1/2,j    sqrt(g)        |i,j-1/2
c
c
c    rho    |  a  |   c  b |
c   ------- |     |  V  V  |
c   sqrt(g) | b c |        |i,j
c
c
c   There are two linearization methods available Newton and Picard.
c
c   Application of Newton linearization:
c
c   a  b    _a  b      a _b     _a _b
c  V  V  =  V  V   +  V  V   -  V  V
c                                                 1
c   results in the following contribution of the V  component to the matrix:
c
c
c    rho      1 |i+1/2,j     rho      2 |i,j+1/2      rho      a |i+1/2,j
c   -------- V  |         + -----    V  |          + -----    V  |         +
c   sqrt(g)     |i-1/2,j    sqrt(g)     |i,j-1/2     sqrt(g)     |i-1/2,j
c
c
c  2 rho    |  a  |   b |
c   ------- |     |  V  |
c   sqrt(g) | 1 b |     |i,j
c
c                                              2
c  and of course a similar expression for the V  component
c
c  Application of the Picard linearization (in the b-th momentum equation)
c
c   a  b     _a  b
c  V  V  =   V  V
c                                               1
c  results in the following contribution of the V  component to the matrix:
c
c
c    rho      1 |i+1/2,j     rho      2 |i,j+1/2      rho    |  a  |   b |
c   -------- V  |         + -----    V  |         +  -----   |     |  V  |
c   sqrt(g)     |i-1/2,j    sqrt(g)     |i,j-1/2     sqrt(g) | 1 b |     |i,j
c
c                                                2
c    and of course a similar expression for the V  component
c
c  In case of Picard the linearization is as follows:
c
c  In the V^1 equation all terms V^2 are taken at the old level,
c  in the V^2 equation all terms V^1 are taken at the old level.
c
c
c  In the conservative approach the term in integration point intp = 1
c  is used in a conservative way i.e.
c
c                                 |(1,0)                             |(2,0)
c  V^1 cell:  rho/sqrt(g) (V^1)^2 |        = 1/2 rho/sqrt(g) (V^1)^2 |
c                                 |(-1,0)                            |(-2,0)
c
c                                 |(0,1)                             |(0,2)
c  V^2 cell:  rho/sqrt(g) (V^2)^2 |        = 1/2 rho/sqrt(g) (V^1)^2 |
c                                 |(0,-1)                            |(0,-2)
c **********************************************************************
c
c                       DATA STATEMENTS
c
      data istwo / 1, 2,
     +             2, 3 /
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      if(mcom.eq.0) then
         mlin     = iindsc(8)
      else if(mcom.eq.1) then
         mlin     = 2
      end if

      iconserv = iindsc(9)
      iupwind  = iindsc(5)

      if ( mlin.eq.1 ) then

c     --- mlin=1, Newton lin. : multiplier is 2d0.

         facs = 2d0

      else

c     --- mlin=2, Picard lin. : multiplier is 1d0.

         facs = 1d0

      end if

      ilow = 1
      iupp = ni
      jlow = 1
      jupp = nj
      if ( alphad.eq.1 ) then

c     --- V^1 stencil

         iupp = ni+1
         if ( intp.eq.1 ) then

c        --- Points (-1,0) and (1,0)

            ilow = 0
            if ( iconserv.eq.1 ) iupp = ni+2
            betad = 1

         else if ( intp.eq.2 ) then

c        --- Points (0,-1) and (0,1)

            jupp = nj+1
            betad = 2
            if ( iupwind.ge.1 .and. iconserv.eq.1 ) then
               jlow = 0
               jupp = nj+2
            end if

         end if

      else if ( alphad.eq.2 ) then

c     --- V^2 stencil

         jupp = nj+1
         if ( intp.eq.1 ) then

c        --- Points (0,-1) and (0,1)

            jlow = 0
            if ( iconserv.eq.1 ) jupp = nj+2
           betad = 2

         else if ( intp.eq.2 ) then

c        --- Points (-1,0) and (1,0)

            iupp = ni+1
            betad = 1
            if ( iupwind.ge.1 .and. iconserv.eq.1 ) then
               ilow = 0
               iupp = ni+2
            end if

         end if

      end if
      if ( intp.eq.0 ) then

c     --- Centroid of cell (0,0)
c         Take care of christoffel symbol term
c         from (convection term),betad derivative in cell centre

         do i = 1, 2
            do jp = jlow, jupp
               do ip = ilow, iupp
                  vsum(ip,jp,i) =
     +                            densrho(ip,jp)*facs*
     +                           (chsym(ip,jp,alphad,istwo(i,1))*
     +                            veloc(ip,jp,1)+
     +                            chsym(ip,jp,alphad,istwo(i,2))*
     +                            veloc(ip,jp,2))
               end do
            end do
         end do

      else if ( intp.eq.1 ) then

c     --- alphad = betad, hence (1,1) or (2,2) element in block matrix

         if ( iconserv.eq.1 .and. iupwind.le.0 ) facs = 0.5d0*facs
         do jp = jlow, jupp
            do ip = ilow, iupp
               vsum(ip,jp,betad) = vsum(ip,jp,betad)+facs*
     +                             densrho(ip,jp)*
     +                             veloc(ip,jp,alphad)
            end do
         end do

      else

c     --- alphad # betad, hence (1,2) and (2,1) element in block matrix

         do jp = jlow, jupp
            do ip = ilow, iupp
               vsum(ip,jp,alphad) = vsum(ip,jp,alphad)+
     +                              densrho(ip,jp)*
     +                              veloc(ip,jp,betad)
            end do
         end do

         if ( mlin.eq.1 ) then

c        --- mlin=1, Newton linearisation. No contribution for
c                       Picard.

            do jp = jlow, jupp
               do ip = ilow, iupp
                  vsum(ip,jp,betad) = vsum(ip,jp,betad)+
     +                                densrho(ip,jp)*
     +                                veloc(ip,jp,alphad)
               end do
            end do

         end if

      end if

      end
