      subroutine ismomst3 ( ni, nj, nk, nvirtual, mu, jctg, chsym, vsum,
     +                      dvsum, alphad, locdx, nzdvsum, iparea )
c======================================================================
c
c        programmer     Ronald van Nooyen
c        version 2.0    date 26-01-95 (Guus: New parameter list)
c        version 1.0    date 14-06-93
c
c   copyright (c) 1993-1995  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c***********************************************************************
c
c
c                 DESCRIPTION
c
c     determines the coefficients of the flux components and the partial
c     derivatives of the flux components in a given integration point
c     within a flux cell for the stress terms
c     Three-dimensional classical ISNaS discretization
c***********************************************************************
c
c                      KEYWORDS
c
c    stress
c    matrix
c    right_hand_side
c ********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS
c
      implicit none
      integer ni, nj, nk, nvirtual, locdx(3), alphad,
     +        iparea(2,3)
      logical nzdvsum(3,3)
      double precision mu(1-nvirtual:ni+1+nvirtual,
     +                    1-nvirtual:nj+1+nvirtual,
     +                    1-nvirtual:nk+1+nvirtual),
     +                 jctg(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,
     +                      1-nvirtual:nk+1+nvirtual,6),
     +                 vsum(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,
     +                      1-nvirtual:nk+1+nvirtual,3),
     +                 dvsum(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,
     +                       1-nvirtual:nk+1+nvirtual,3,3),
     +                 chsym(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,
     +                       1-nvirtual:nk+1+nvirtual,3,6)

c     alphad    i    Direction index of integration.
c                    Hence if a V1-cell is used, alphad = 1 and so on
c     chsym     i    Christoffel symbols of the second kind in the
c                    point given by pnttyp
c                    (symmetric contravariant part stored as vector
c                    with index encoding given by istwo)
c     dvsum     o    Flux derivative contributions to the matrix
c     iparea    i    Area of integration points where a contribution to
c                    the matrix is to be calculated
c     jctg      i    Jacobian times symmetric contravariant metric
c                    tensor in point given by pnttyp
c                    (stored as vector index encoding given by istwo)
c     locdx     i    Position of the integration point with respect to
c                    the centre of the integration cell
c                    The cell sizes are assumed to be 2 times 2, hence the
c                    distance of a midside point to a cell centre is always 1
c     mu        i    The laminar viscosity
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     nzdvsum   i    Indicates whether or not a partial derivative in a
c                    certain direction occurs in the stencil.
c                    For example the derivative dU2/dksi1 in the point (0,1)
c                    or the V1-stencil of the 13-point molecule does not exist
c                    nzdvsum should therefore be false in that point
c     vsum      o    Flux contributions to the matrix
c**********************************************************************
c
c                 COMMON BLOCKS
c
c**********************************************************************
c
c                 LOCAL PARAMETERS
c

      integer ip, jp, kp, betad, gammad, i, j, iptensor1,
     +        iptensor2, iptensor3, iptensor4, iptensor5, iptensor6,
     +        istwo(3,3)
      double precision sign

      data istwo / 1, 2, 3,
     +             2, 4, 5,
     +             3, 5, 6 /

c     betad     Direction index, usually the direction perpendicular to the
c               integration direction of the boundary integral.
c               For example betad in local point (1,0) is 1 and in (0,1): 2
c     gammad    Direction index
c     i         General loop variable
c     ip        Local counting variable i-direction
c     iptensor1 Tensor index
c     iptensor2 Tensor index
c     iptensor3 Tensor index
c     iptensor4 Tensor index
c     iptensor5 Tensor index
c     iptensor6 Tensor index
c     istwo     Translation table to store and retrieve the
c               elements of a symmetric ndim-dimensional
c               rank 2 tensor from a vector with length
c               (ndim*(ndim+1))/2
c     j         General loop variable
c     jp        Local counting variable j-direction
c     kp        Local counting variable k-direction
c     sign      sign for multiplication (1 or -1)
c**********************************************************************
c
c                 I/O
c
c**********************************************************************
c
c                 SUBROUTINES CALLED
c
c**********************************************************************
c
c                 ERROR MESSAGES
c
c***********************************************************************
c
c     PSEUDO CODE
c
c       Relevant formula
c
c
c    The stress terms in case of the classical discretization are given by:
c
c
c             a1 |i+1/2,j,k              a2 |i,j+1/2,k
c   sqrt(g)  T   |           + sqrt(g)  T   |          +
c                |i-1/2,j,k                 |i,j-1/2,k
c
c
c              a3 |i,j,k+1/2              |  a  |   cb |
c     sqrt(g) T   |           +   sqrt(g) |     |  T   |
c                 |i,j,k-1/2              | b c |      |i,j,k
c
c             ab
c   The term T   can be written as:
c
c
c    ab         ac b      bc a
c  -T   = mu ( g  U    + g  U   )
c                  ,c        ,c
c
c           aa         ac a
c   Hence -T   = 2 mu g  U
c                         ,c
c======================================================================
c
c     --- In first instance we consider all coefficients before the
c         velocity derivative terms DVa/Dksib

      if ( abs(locdx(1))+abs(locdx(2))+abs(locdx(3)).eq.1 ) then

c     --- Integration points (0,1,0), (0,-1,0), (1,0,0), (-1,0,0),
c                            (0,0,1), (0,0,-1)

         if (abs(locdx(1)).ne.0) betad = 1
         if (abs(locdx(2)).ne.0) betad = 2
         if (abs(locdx(3)).ne.0) betad = 3
         sign = -locdx(betad)
         if ( alphad.eq.betad ) then

c        --- alpha = beta

            do 130 gammad = 1, 3

               iptensor1 = istwo(gammad,alphad)
               do 120 kp = iparea(1,3), iparea(2,3)
                  do 110 jp = iparea(1,2), iparea(2,2)
                     do 100 ip = iparea(1,1), iparea(2,1)
                        dvsum(ip,jp,kp,alphad,gammad) =
     +                       sign*2d0*mu(ip,jp,kp)*
     +                       jctg(ip,jp,kp,iptensor1)
100                  continue
110               continue
120            continue


               nzdvsum(alphad,gammad) = .true.

130         continue
c
         else

c        --- alpha # beta

            do 190 gammad = 1, 3
               iptensor1 = istwo(gammad,alphad)
               iptensor2 = istwo(gammad,betad)
               do 180 kp = iparea(1,3), iparea(2,3)
                  do 170 jp = iparea(1,2), iparea(2,2)
                     do 160 ip = iparea(1,1), iparea(2,1)
                        dvsum(ip,jp,kp,betad,gammad) =
     +                       sign*mu(ip,jp,kp)*
     +                       jctg(ip,jp,kp,iptensor1)
                        dvsum(ip,jp,kp,alphad,gammad) =
     +                       sign*mu(ip,jp,kp)*
     +                       jctg(ip,jp,kp,iptensor2)
160                  continue
170               continue
180            continue

               nzdvsum(alphad,gammad) = .true.
               nzdvsum(betad,gammad) = .true.

190         continue

         end if

         else if ( locdx(1).eq.0 .and. locdx(2).eq.0 .and.
     +             locdx(3).eq.0 ) then

c        --- Centroid point (0,0,0)
c            This concerns the terms involving the Christoffel symbols

         do 240 gammad = 1, 3
            do 230 j = 1, 3

               iptensor1 = istwo(j,1)
               iptensor2 = istwo(j,2)
               iptensor3 = istwo(j,3)
               iptensor4 = istwo(gammad,1)
               iptensor5 = istwo(gammad,2)
               iptensor6 = istwo(gammad,3)
               do 220 kp = iparea(1,3), iparea(2,3)
                  do 210 jp = iparea(1,2), iparea(2,2)
                     do 200 ip = iparea(1,1), iparea(2,1)
                        dvsum(ip,jp,kp,j,gammad)  = - 2d0*mu(ip,jp,kp)*
     +                     ( chsym(ip,jp,kp,alphad,iptensor1)*
     +                       jctg(ip,jp,kp,iptensor4)+
     +                       chsym(ip,jp,kp,alphad,iptensor2)*
     +                       jctg(ip,jp,kp,iptensor5) +
     +                       chsym(ip,jp,kp,alphad,iptensor3)*
     +                       jctg(ip,jp,kp,iptensor6) )
200                  continue
210               continue
220            continue
               nzdvsum(j,gammad) = .true.
230         continue

240      continue

      end if

c     --- Finally all coefficients before Ua

      do 340 i = 1, 3

         do 330 j = 1, 3

            if ( nzdvsum(i,j) ) then

c           --- Point is in stencil

               iptensor1 = istwo(j,1)
               iptensor2 = istwo(j,2)
               iptensor3 = istwo(j,3)

               do 320 kp = iparea(1,3),iparea(2,3)
                  do 310 jp = iparea(1,2),iparea(2,2)
                     do 300 ip = iparea(1,1),iparea(2,1)
                        vsum(ip,jp,kp,1) = vsum(ip,jp,kp,1)+
     +                       dvsum(ip,jp,kp,i,j)*
     +                       chsym(ip,jp,kp,i,iptensor1)
                        vsum(ip,jp,kp,2) = vsum(ip,jp,kp,2)+
     +                       dvsum(ip,jp,kp,i,j)*
     +                       chsym(ip,jp,kp,i,iptensor2)
                        vsum(ip,jp,kp,3) = vsum(ip,jp,kp,3)+
     +                       dvsum(ip,jp,kp,i,j)*
     +                       chsym(ip,jp,kp,i,iptensor3)
300                  continue
310               continue
320            continue

            end if
330      continue
340   continue

      end

