      subroutine ispndf1 ( action, compon, vartyp, pnttyp, ioffst,
     +                     factor, split, ndim )
c ======================================================================
c
c
c        programmer     Ronald van Nooyen
c        version 1.1    date 13-12-1994 (New layout)
c        version 1.0    date 24-06-1994
c
c   copyright (c) 1994-1994  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c ********************************************************************
c
c                         DESCRIPTION
c
c     Make tables for interpolation and finite difference formulas in one
c     direction only
c     By applying it to the components 1..ndim of a position vector
c     an n-dimensional interpolation, extrapolation or finite
c     difference is constructed in ispndf.
c ********************************************************************
c
c                      KEYWORDS
c
c     finite_difference_formulas
c     interpolation_formulas
c ********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS
c
      implicit none
      logical split
      integer action, compon, vartyp, pnttyp, ndim
      integer ioffst(2)
      double precision factor(2)

c     action    i    Indicates the type of action to be carried out
c                    Possible values:
c                    0:   interpolation or extrapolation for point value
c                    1 to ndim: interpolation or extrapolation for
c                         partial derivative in direction action
c     compon    i    Co-ordinate direction with respect to the action
c     factor    i    factors for linear interpolation or
c                    difference formula in direction compon
c     ioffst    i    Contains the offsets for the linear interpolation
c                    or difference formula with respect to the global
c                    numbering
c     ndim      i    Dimension of space (2 or 3)
c     pnttyp    i    reserved
c     split     i    If true, two points are needed for the formula,
c                    otherwise only one is needed
c     vartyp    i    Variable to be interpolated/extrapolated
c                    vartyp: 1 for V1, 2 for V2 and 3 for pressure,
c                    0 for vertex (for ndim=2)
c                    or :
c                    vartyp: 1 for V1, 2 for V2 3, for V3 and
c                    4 for pressure, 0 for vertex  (for ndim=3)
c ********************************************************************
c
c                    COMMON BLOCKS
c
c ********************************************************************
c
c                    LOCAL PARAMETERS

      integer crdiff, crcomp

c     crcomp  Local coordinate of point with respect to vertex in the
c             direction compon
c     crdiff  Local difference between point indicated by pnttyp, i.e.
c             point where the interpolation must be carried out and
c             point where the unknown indicated by vartype is present
c             within the same co-ordinate system in the direction compon
c ********************************************************************
c
c             SUBROUTINES CALLED
c
c ********************************************************************
c
c                              I/O
c
c ********************************************************************
c
c                    ERROR MESSAGES
c
c ********************************************************************
c
c                    METHOD
c
c     Essential in this subroutine is the realtion between local and
c     global numbering in ISNaS
c     In R2 all 4 points in left under corner have the same cell index i and j
c     In R3 the same is true for 8 points
c
c     The local co-ordinates for the points with the same i,j index
c     are given below for R2, the extension to R3 can be found in pnttype
c
c                       ksi^2
c                        ^     +----|----+
c                        |     |         |
c                              |         |
c                             --    *    --
c                            3 |   4     |
c                              |         |
c                              +----|----+   -> ksi^1
c                             1     2

c     The local coordinate crcomp of the point pnttyp in the direction compon
c     is given in the following table:
c
c     point      2D                        3D
c             x      y              x      y      z - direction
c     1       0      0              0      0      0
c     2       1      0              1      0      0
c     3       0      1              0      1      0
c     4       1      1              1      1      0
c     5                             0      0      1
c     6                             1      0      1
c     7                             0      1      1
c     8                             1      1      1
c
c     The local differences crdiff between point pntype and point where
c     variable is present depend on pnttyp, compon and vartyp
c     If the unknown corresponds to a vertex (vartyp=0) crdiff = crcomp,
c     since for example
c     interpolation in the x-direction in points 1 and 3 is not necessary,
c     whereas in points 2 and 4 the distance to the vertex in x-direction is 1
c     If the unknown corresponds to a centroid (vartyp=ndim+1)
c     crdiff = crcomp-1, since for example
c     interpolation in the x-direction in points 2 and 4 is not necessary,
c     whereas in points 1 and 3 the distance to the centroid in x-direction
c     is equal to -1
c     In the same way it is clear that if compon is equal to vartyp,
c     crdiff = crcomp and otherwise crdiff = crcomp-1
c
c     In factor the weights for the linear interpolation or finite difference
c     formulas are stored, in ioffst the corresponding offsets.
c     The offset is computed with respect to the global numbering.
c     For example linear interpolation in the x-direction of vertex unknowns
c     in point 2 (2D) gives
c         v(int) = (v(0,0) + v(2,0))/2
c     Hence factor(1) = 0.5,  factor(2) = 0.5
c           ioffst(1) = 0,    ioffst(2) = 1
c     since v(0,0) has the same global number as point 2 and the cell number
c     of (2,0) in x-direction is one higher than that of point 2
c     In the same way gives a central difference of V2 in point 2 in
c     x-direction:
c         V2(diff_x) = (V2(3,0) - V2(-1,0))/2
c     Hence factor(1) = -0.5,  factor(2) = 0.5
c           ioffst(1) = -1,    ioffst(2) = 1
c     and a central difference of V1 in point 2 in x-direction:
c         V1(diff_x) = V1(2,0) - V1(0,0)
c     Hence factor(1) = -1,  factor(2) = 1
c           ioffst(1) = 0,   ioffst(2) = 1
c
c
c                    PSEUDO CODE
c
c     Determine component crcomp for direction compon of the local coordinate
c     vector for pnttyp
c     Determine component for direction compon of the local
c     coordinate vector for pnttyp relative to the point
c     where vartyp is known
c     Fill ioffst and factor
c ********************************************************************
c
c                    DATA STATEMENTS
c
c ======================================================================
c

c     --- Determine component for direction compon of the local
c         coordinate vector for pnttyp

      crcomp = mod( (pnttyp-1)/(2**(compon-1)), 2)

c     --- Determine component for direction compon of the local
c         coordinate vector for pnttyp relative to the point
c         where vartyp is known

      if (vartyp.eq.0) then

c     --- crcomp itself is the coordinate w.r.t. the vertex

         crdiff = crcomp

      else if (vartyp.eq.ndim+1) then

c     --- Variable given in cell centre, local
c         coordinates of cell centre: (1,..,1), so

         crdiff = crcomp-1

      else if ( vartyp.eq.compon ) then

c        --- Flux variable given in the centre of the cell side
c            orthogonal to direction vartyp

         crdiff = crcomp

      else if (vartyp.ge.1 .and. vartyp.le.ndim .and.
     +        vartyp.ne.compon) then

c        --- Flux variable given in the centre of the cell side
c            orthogonal to direction vartyp and vartyp<>compon

         crdiff = crcomp-1

      end if

      if ( action.eq.compon ) then

c     --- Determine finite difference for dependent variable vartyp
c         in point pnttyp along component direction compon:
c         two points needed

         split = .true.

         if ( crdiff.eq.0 ) then

c        --- Finite difference for point where the dependent variable
c            is known. Assume both neighbours exist, use central
c            difference:

            ioffst(1) = -1
            factor(1) = -5d-1
            ioffst(2) =  1
            factor(2) =  5d-1

         else

c        --- Finite difference for point where the dependent variable
c            is not known. Assume both neighbours exist, use central
c            difference:

            ioffst(1) =  min(0,crdiff)
            factor(1) = -1d0
            ioffst(2) =  max(0,crdiff)
            factor(2) =  1d0

         end if

      else

c     --- Determine value of dependent variable vartyp in point pnttyp
c         If there is a coordinate difference in this direction we need
c         to use two points for this direction

         split = (crdiff.ne.0)

         if (.not. split) then

c        --- No interpolation needed for this component

            ioffst(1) =  0
            factor(1) =  1d0
            ioffst(2) =  0
            factor(2) =  0d0

         else

c        --- Interpolation needed for this component. Assume both
c            neighbours exist, use symmetric average:

            ioffst(1) =  min(0,crdiff)
            factor(1) =  5d-1
            ioffst(2) =  max(0,crdiff)
            factor(2) =  5d-1

         end if

      end if

      end
