      subroutine isurhs3d ( alphad, ni, nj, nk, nvirtual, rhside, vsum,
     +                      veloc, solut, iarea, dcell, locdx,
     +                      ichoisdis, kappa, phi, mbound, alphapl )
c ======================================================================
c
c        programmer     Marcel Zijlema
c        version 2.1    date   17-02-1997 Add new classes of limiters, part II
c        version 2.0    date   09-12-1996 Add new classes of limiters, part I
c        version 1.0    date   26-03-1996
c
c   copyright (c) 1996-1997  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Update right-hand side vector of the momentum equation with
c     higher order upwind scheme through defect correction approach in 3D
c **********************************************************************
c
c                       KEYWORDS
c
c     discretization
c     right_hand_side
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nk, nvirtual, iarea(2,3), dcell(3), locdx(3),
     +        alphad, ichoisdis
      double precision kappa, phi, mbound, alphapl,
     +                 rhside(1:ni+1,1:nj+1,1:nk+1),
     +                 solut(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,
     +                       1-nvirtual:nk+1+nvirtual),
     +                 veloc(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,
     +                       1-nvirtual:nk+1+nvirtual,1:3),
     +                 vsum(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,
     +                      1-nvirtual:nk+1+nvirtual,1:3)

c     alphad    i    Direction index of integration.
c                    Hence if a V1-cell is used, alphad = 1 and so on
c     alphapl   i    Indicate a parameter of the PL-kappa, MPL1-kappa and
c                    MPL2-kappa limiters
c     dcell     i    reserved
c     iarea     i    Area of arrays
c     ichoisdis i    Choice parameter with respect to the convection scheme
c                    for discretization of convective term of momentum and
c                    transport equations.
c                    Meaning of ichoisdis:
c                      ichoisdis =  0: Second order central scheme
c                      ichoisdis =  1: First order upwind scheme
c                      ichoisdis =  2: Hybrid central/upwind scheme
c                      ichoisdis =  3: Higher order upwind scheme (kappa-scheme)
c                      ichoisdis =  4: Sweby Phi-limiter
c                      ichoisdis =  5: R-kappa limiter
c                      ichoisdis =  6: MR-kappa limiter
c                      ichoisdis =  7: Symmetric Ratio limiter
c                      ichoisdis =  8: PL-kappa limiter
c                      ichoisdis =  9: MPL1-kappa limiter
c                      ichoisdis = 10: MPL2-kappa limiter
c                      ichoisdis = 11: Symmetric PL-kappa limiter
c     kappa     i    Controls the spatial accuracy of the higher order
c                    schemes obtained by kappa-formulation
c     locdx     i    Position of the integration point with respect to
c                    the centre of the integration cell
c                    The cell sizes are assumed to be 2 times 2, hence the
c                    distance of a midside point to a cell centre is always 1
c     mbound    i    Gives the maximum bound of the flux limiter
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     phi       i    Indicate a parameter of the Sweby Phi-limiter
c     rhside   i/o   reserved
c     solut     i    reserved
c     veloc     i    Array for the old velocities (used in convection
c                    term calculation)
c     vsum      i    Flux contributions to the matrix
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ishift, jshift, kshift, ip, jp, kp, dx(3), betad
      double precision flux, r, fact, phi0, phi1, phi2, xlim

c     betad     Direction index, usually the direction perpendicular to the
c               integration direction of the boundary integral.
c               For example betad in local point (1,0) is 1 and in (0,1): 2
c     dx        Point location in cell relative to local
c               coordinates with (0,0) and (2,2) or
c               (0,0,0) and (2,2,2) as opposite cell corners
c     fact      Multiplication factor
c     flux      Contravariant flux component
c     ip        Local counting variable i-direction
c     ishift    Shift in i-direction
c     jp        Local counting variable j-direction
c     jshift    Shift in j-direction
c     kp        Local counting variable k-direction
c     kshift    Shift in k-direction
c     phi0      Value of transport quantity in centroid
c     phi1      Value of transport quantity in a shifted point
c     phi2      Value of transport quantity in a shifted point
c     r         Ratio of consecutive solution gradients
c     xlim      Limiter
c **********************************************************************
c
c                       I/O
c
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c   The higher order upwind schemes are treated by defect correction in the
c   following way:
c
c                los         hos        los  o
c   phi_i = phi_i   + ( phi_i   -  phi_i    )
c
c   where phi_i denotes the flux at an integration point, 'hos' means the
c   higher order upwind scheme, 'los' the standard first order upwind
c   scheme and 'o' denotes the preceding time-level.
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
c     --- Shift from integration cell centre to cell containing the
c         integration point

      ishift = dcell(1)
      jshift = dcell(2)
      kshift = dcell(3)

      if ( abs(locdx(1))+abs(locdx(2))+abs(locdx(3)).eq.1 ) then

c     --- centres of integration cell sides
c         betad in (convection term), betad

        if ( locdx(1).ne.0 ) then
           betad = 1
        else if ( locdx(2).ne.0 ) then
           betad = 2
        else
           betad = 3
        end if

        dx(1) = 0
        dx(2) = 0
        dx(3) = 0
        dx(betad) = 1

        if ( ichoisdis.eq.3 ) then

c       --- ichoisdis = 3, Higher order upwind scheme (kappa-scheme)

          do kp = iarea(1,3)+1, iarea(2,3)-1
            do jp = iarea(1,2)+1, iarea(2,2)-1
              do ip = iarea(1,1)+1, iarea(2,1)-1

c             --- Compute the flux at the integration point and transport
c                 quantities in centroid and shifted points

                flux = veloc(ip+ishift,jp+jshift,kp+kshift,betad)
                phi0 = solut(ip+ishift,jp+jshift,kp+kshift)
                phi1 = solut(ip+ishift+dx(1),jp+jshift+dx(2),
     +                       kp+kshift+dx(3))

c               --- update right-hand side using the defect correction
c                   technique

                if ( flux.gt.0d0 ) then
                  phi2 = solut(ip+ishift-dx(1),jp+jshift-dx(2),
     +                         kp+kshift-dx(3))
                  r = ( phi1 - phi0 + 1d-10) / ( phi0 - phi2 + 1d-10 )
                  xlim = 0.5d0*(1d0+kappa)*r + 0.5d0*(1d0-kappa)
                  rhside(ip,jp,kp) = rhside(ip,jp,kp) - 0.5d0 *
     +                               vsum(ip+ishift,jp+jshift,kp+kshift,
     +                                    alphad) *
     +                               xlim * ( phi0 - phi2 )
                else
                  phi2 = solut(ip+ishift+2*dx(1),jp+jshift+2*dx(2),
     +                         kp+kshift+2*dx(3))
                  r  = ( phi1 - phi0 + 1d-10) / ( phi2 - phi1 + 1d-10 )
                  xlim = 0.5d0*(1d0+kappa)*r + 0.5d0*(1d0-kappa)
                  rhside(ip,jp,kp) = rhside(ip,jp,kp) + 0.5d0 *
     +                               vsum(ip+ishift,jp+jshift,kp+kshift,
     +                                    alphad) *
     +                               xlim * ( phi2 - phi1 )

                end if
              end do
            end do
          end do

        else if ( ichoisdis.eq.4 ) then

c       --- ichoisdis = 4, Sweby Phi-limiter

          do kp = iarea(1,3)+1, iarea(2,3)-1
            do jp = iarea(1,2)+1, iarea(2,2)-1
              do ip = iarea(1,1)+1, iarea(2,1)-1

c             --- Compute the flux at the integration point and transport
c                 quantities in centroid and shifted points

                flux = veloc(ip+ishift,jp+jshift,kp+kshift,betad)
                phi0 = solut(ip+ishift,jp+jshift,kp+kshift)
                phi1 = solut(ip+ishift+dx(1),jp+jshift+dx(2),
     +                       kp+kshift+dx(3))

c               --- update right-hand side using the defect correction
c                   technique

                if ( flux.gt.0d0 ) then
                  phi2 = solut(ip+ishift-dx(1),jp+jshift-dx(2),
     +                         kp+kshift-dx(3))
                  r = ( phi1 - phi0 + 1d-10) / ( phi0 - phi2 + 1d-10 )
                  xlim = max( max( 0d0, min(phi*r,1d0) ), min(r, phi) )
                  rhside(ip,jp,kp) = rhside(ip,jp,kp) - 0.5d0 *
     +                               vsum(ip+ishift,jp+jshift,kp+kshift,
     +                                    alphad) *
     +                               xlim * ( phi0 - phi2 )
                else
                  phi2 = solut(ip+ishift+2*dx(1),jp+jshift+2*dx(2),
     +                         kp+kshift+2*dx(3))
                  r  = ( phi1 - phi0 + 1d-10) / ( phi2 - phi1 + 1d-10 )
                  xlim = max( max( 0d0, min(phi*r,1d0) ), min(r, phi) )
                  rhside(ip,jp,kp) = rhside(ip,jp,kp) + 0.5d0 *
     +                               vsum(ip+ishift,jp+jshift,kp+kshift,
     +                                    alphad) *
     +                               xlim * ( phi2 - phi1 )

                end if
              end do
            end do
          end do

        else if ( ichoisdis.eq.5 ) then

c       --- ichoisdis = 5, R-kappa limiter

          do kp = iarea(1,3)+1, iarea(2,3)-1
            do jp = iarea(1,2)+1, iarea(2,2)-1
              do ip = iarea(1,1)+1, iarea(2,1)-1

c             --- Compute the flux at the integration point and transport
c                 quantities in centroid and shifted points

                flux = veloc(ip+ishift,jp+jshift,kp+kshift,betad)
                phi0 = solut(ip+ishift,jp+jshift,kp+kshift)
                phi1 = solut(ip+ishift+dx(1),jp+jshift+dx(2),
     +                       kp+kshift+dx(3))

c               --- update right-hand side using the defect correction
c                   technique

                if ( flux.gt.0d0 ) then
                  phi2 = solut(ip+ishift-dx(1),jp+jshift-dx(2),
     +                         kp+kshift-dx(3))
                  r = ( phi1 - phi0 + 1d-10) / ( phi0 - phi2 + 1d-10 )
                  if ( kappa.lt.0d0 ) then
                    if ( r.le.1d0 ) then
                      xlim = (r + abs(r)) * (-r*r + (3d0+kappa)*r
     +                        -kappa)/((1d0+r)**2)
                    else
                      xlim = ((2d0+kappa)*r-kappa)/(1d0+r)
                    end if
                  else
                    xlim = (r + abs(r)) * ((1d0+kappa)*r +
     +                     1d0-kappa)/((1d0+r)**2)
                  end if
                  rhside(ip,jp,kp) = rhside(ip,jp,kp) - 0.5d0 *
     +                               vsum(ip+ishift,jp+jshift,kp+kshift,
     +                                    alphad) *
     +                               xlim * ( phi0 - phi2 )
                else
                  phi2 = solut(ip+ishift+2*dx(1),jp+jshift+2*dx(2),
     +                         kp+kshift+2*dx(3))
                  r  = ( phi1 - phi0 + 1d-10) / ( phi2 - phi1 + 1d-10 )
                  if ( kappa.lt.0d0 ) then
                    if ( r.le.1d0 ) then
                      xlim = (r + abs(r)) * (-r*r + (3d0+kappa)*r
     +                        -kappa)/((1d0+r)**2)
                    else
                      xlim = ((2d0+kappa)*r-kappa)/(1d0+r)
                    end if
                  else
                    xlim = (r + abs(r)) * ((1d0+kappa)*r +
     +                     1d0-kappa)/((1d0+r)**2)
                  end if
                  rhside(ip,jp,kp) = rhside(ip,jp,kp) + 0.5d0 *
     +                               vsum(ip+ishift,jp+jshift,kp+kshift,
     +                                    alphad) *
     +                               xlim * ( phi2 - phi1 )

                end if
              end do
            end do
          end do

        else if ( ichoisdis.eq.6 ) then

c       --- ichoisdis = 6, MR-kappa limiter

          do kp = iarea(1,3)+1, iarea(2,3)-1
            do jp = iarea(1,2)+1, iarea(2,2)-1
              do ip = iarea(1,1)+1, iarea(2,1)-1

c             --- Compute the flux at the integration point and transport
c                 quantities in centroid and shifted points

                flux = veloc(ip+ishift,jp+jshift,kp+kshift,betad)
                phi0 = solut(ip+ishift,jp+jshift,kp+kshift)
                phi1 = solut(ip+ishift+dx(1),jp+jshift+dx(2),
     +                       kp+kshift+dx(3))

c               --- update right-hand side using the defect correction
c                   technique

                if ( flux.gt.0d0 ) then
                  phi2 = solut(ip+ishift-dx(1),jp+jshift-dx(2),
     +                         kp+kshift-dx(3))
                  r = ( phi1 - phi0 + 1d-10) / ( phi0 - phi2 + 1d-10 )
                  if ( r.le.1d0/3d0 ) then
                    xlim = r + abs(r)
                  else if ( r.le.1d0 ) then
                    xlim = ((8d0+9d0*kappa)*r*r+(2d0-12d0*kappa)*r+
     +                      2d0+3d0*kappa)/(3d0*(1d0+r)**2)
                  else
                    if ( kappa.lt.0d0 ) then
                      xlim = ((2d0+kappa)*r-kappa)/(1d0+r)
                    else
                      xlim = ((8d0+9d0*kappa)*r*r+(2d0-12d0*kappa)*r+
     +                        2d0+3d0*kappa)/(3d0*(1d0+r)**2)
                    end if
                  end if
                  rhside(ip,jp,kp) = rhside(ip,jp,kp) - 0.5d0 *
     +                               vsum(ip+ishift,jp+jshift,kp+kshift,
     +                                    alphad) *
     +                               xlim * ( phi0 - phi2 )
                else
                  phi2 = solut(ip+ishift+2*dx(1),jp+jshift+2*dx(2),
     +                         kp+kshift+2*dx(3))
                  r  = ( phi1 - phi0 + 1d-10) / ( phi2 - phi1 + 1d-10 )
                  if ( r.le.1d0/3d0 ) then
                    xlim = r + abs(r)
                  else if ( r.le.1d0 ) then
                    xlim = ((8d0+9d0*kappa)*r*r+(2d0-12d0*kappa)*r+
     +                      2d0+3d0*kappa)/(3d0*(1d0+r)**2)
                  else
                    if ( kappa.lt.0d0 ) then
                      xlim = ((2d0+kappa)*r-kappa)/(1d0+r)
                    else
                      xlim = ((8d0+9d0*kappa)*r*r+(2d0-12d0*kappa)*r+
     +                        2d0+3d0*kappa)/(3d0*(1d0+r)**2)
                    end if
                  end if
                  rhside(ip,jp,kp) = rhside(ip,jp,kp) + 0.5d0 *
     +                               vsum(ip+ishift,jp+jshift,kp+kshift,
     +                                    alphad) *
     +                               xlim * ( phi2 - phi1 )

                end if
              end do
            end do
          end do

        else if ( ichoisdis.eq.7 ) then

c       --- ichoisdis = 7, Symmetric Ratio limiter

          do kp = iarea(1,3)+1, iarea(2,3)-1
            do jp = iarea(1,2)+1, iarea(2,2)-1
              do ip = iarea(1,1)+1, iarea(2,1)-1

c             --- Compute the flux at the integration point and transport
c                 quantities in centroid and shifted points

                flux = veloc(ip+ishift,jp+jshift,kp+kshift,betad)
                phi0 = solut(ip+ishift,jp+jshift,kp+kshift)
                phi1 = solut(ip+ishift+dx(1),jp+jshift+dx(2),
     +                       kp+kshift+dx(3))

c               --- update right-hand side using the defect correction
c                   technique

                if ( flux.gt.0d0 ) then
                  phi2 = solut(ip+ishift-dx(1),jp+jshift-dx(2),
     +                         kp+kshift-dx(3))
                  r = ( phi1 - phi0 + 1d-10) / ( phi0 - phi2 + 1d-10 )
                  xlim = mbound*(r*r+r)/(r*r+2d0*(mbound-1d0)*r+1d0)
                  rhside(ip,jp,kp) = rhside(ip,jp,kp) - 0.5d0 *
     +                               vsum(ip+ishift,jp+jshift,kp+kshift,
     +                                    alphad) *
     +                               xlim * ( phi0 - phi2 )
                else
                  phi2 = solut(ip+ishift+2*dx(1),jp+jshift+2*dx(2),
     +                         kp+kshift+2*dx(3))
                  r  = ( phi1 - phi0 + 1d-10) / ( phi2 - phi1 + 1d-10 )
                  xlim = mbound*(r*r+r)/(r*r+2d0*(mbound-1d0)*r+1d0)
                  rhside(ip,jp,kp) = rhside(ip,jp,kp) + 0.5d0 *
     +                               vsum(ip+ishift,jp+jshift,kp+kshift,
     +                                    alphad) *
     +                               xlim * ( phi2 - phi1 )

                end if
              end do
            end do
          end do

        else if ( ichoisdis.eq.8 ) then

c       --- ichoisdis = 8, PL-kappa limiter

          do kp = iarea(1,3)+1, iarea(2,3)-1
            do jp = iarea(1,2)+1, iarea(2,2)-1
              do ip = iarea(1,1)+1, iarea(2,1)-1

c             --- Compute the flux at the integration point and transport
c                 quantities in centroid and shifted points

                flux = veloc(ip+ishift,jp+jshift,kp+kshift,betad)
                phi0 = solut(ip+ishift,jp+jshift,kp+kshift)
                phi1 = solut(ip+ishift+dx(1),jp+jshift+dx(2),
     +                       kp+kshift+dx(3))

c               --- update right-hand side using the defect correction
c                   technique

                if ( flux.gt.0d0 ) then
                  phi2 = solut(ip+ishift-dx(1),jp+jshift-dx(2),
     +                         kp+kshift-dx(3))
                  r = ( phi1 - phi0 + 1d-10) / ( phi0 - phi2 + 1d-10 )
                  fact = min( mbound,
     +                        0.5d0*(1d0+kappa)*r + 0.5d0*(1d0-kappa) )
                  xlim = max( 0d0, min((2d0+alphapl)*r,fact) )
                  rhside(ip,jp,kp) = rhside(ip,jp,kp) - 0.5d0 *
     +                               vsum(ip+ishift,jp+jshift,kp+kshift,
     +                                    alphad) *
     +                               xlim * ( phi0 - phi2 )
                else
                  phi2 = solut(ip+ishift+2*dx(1),jp+jshift+2*dx(2),
     +                         kp+kshift+2*dx(3))
                  r  = ( phi1 - phi0 + 1d-10) / ( phi2 - phi1 + 1d-10 )
                  fact = min( mbound,
     +                        0.5d0*(1d0+kappa)*r + 0.5d0*(1d0-kappa) )
                  xlim = max( 0d0, min((2d0+alphapl)*r,fact) )
                  rhside(ip,jp,kp) = rhside(ip,jp,kp) + 0.5d0 *
     +                               vsum(ip+ishift,jp+jshift,kp+kshift,
     +                                    alphad) *
     +                               xlim * ( phi2 - phi1 )

                end if
              end do
            end do
          end do

        else if ( ichoisdis.eq.9 ) then

c       --- ichoisdis = 9, MPL1-kappa limiter

          do kp = iarea(1,3)+1, iarea(2,3)-1
            do jp = iarea(1,2)+1, iarea(2,2)-1
              do ip = iarea(1,1)+1, iarea(2,1)-1

c             --- Compute the flux at the integration point and transport
c                 quantities in centroid and shifted points

                flux = veloc(ip+ishift,jp+jshift,kp+kshift,betad)
                phi0 = solut(ip+ishift,jp+jshift,kp+kshift)
                phi1 = solut(ip+ishift+dx(1),jp+jshift+dx(2),
     +                       kp+kshift+dx(3))

c               --- update right-hand side using the defect correction
c                   technique

                if ( flux.gt.0d0 ) then
                  phi2 = solut(ip+ishift-dx(1),jp+jshift-dx(2),
     +                         kp+kshift-dx(3))
                  r = ( phi1 - phi0 + 1d-10) / ( phi0 - phi2 + 1d-10 )
                  fact = min( mbound,
     +                        0.5d0*(1d0+kappa)*r + 0.5d0*(1d0-kappa) )
                  fact = max( min(2d0*r,2d0/3d0), min((2d0+alphapl)*r,
     +                        fact) )
                  xlim = max( 0d0, fact )
                  rhside(ip,jp,kp) = rhside(ip,jp,kp) - 0.5d0 *
     +                               vsum(ip+ishift,jp+jshift,kp+kshift,
     +                                    alphad) *
     +                               xlim * ( phi0 - phi2 )
                else
                  phi2 = solut(ip+ishift+2*dx(1),jp+jshift+2*dx(2),
     +                         kp+kshift+2*dx(3))
                  r  = ( phi1 - phi0 + 1d-10) / ( phi2 - phi1 + 1d-10 )
                  fact = min( mbound,
     +                        0.5d0*(1d0+kappa)*r + 0.5d0*(1d0-kappa) )
                  fact = max( min(2d0*r,2d0/3d0), min((2d0+alphapl)*r,
     +                        fact) )
                  xlim = max( 0d0, fact )
                  rhside(ip,jp,kp) = rhside(ip,jp,kp) + 0.5d0 *
     +                               vsum(ip+ishift,jp+jshift,kp+kshift,
     +                                    alphad) *
     +                               xlim * ( phi2 - phi1 )

                end if
              end do
            end do
          end do

        else if ( ichoisdis.eq.10) then

c       --- ichoisdis = 10, MPL2-kappa limiter

          do kp = iarea(1,3)+1, iarea(2,3)-1
            do jp = iarea(1,2)+1, iarea(2,2)-1
              do ip = iarea(1,1)+1, iarea(2,1)-1

c             --- Compute the flux at the integration point and transport
c                 quantities in centroid and shifted points

                flux = veloc(ip+ishift,jp+jshift,kp+kshift,betad)
                phi0 = solut(ip+ishift,jp+jshift,kp+kshift)
                phi1 = solut(ip+ishift+dx(1),jp+jshift+dx(2),
     +                       kp+kshift+dx(3))

c               --- update right-hand side using the defect correction
c                   technique

                if ( flux.gt.0d0 ) then
                  phi2 = solut(ip+ishift-dx(1),jp+jshift-dx(2),
     +                         kp+kshift-dx(3))
                  r = ( phi1 - phi0 + 1d-10) / ( phi0 - phi2 + 1d-10 )
                  fact = min( mbound,
     +                        0.5d0*(1d0+kappa)*r + 0.5d0*(1d0-kappa) )
                  xlim = max( 0d0, min((2d0+alphapl)*r,fact) ) +
     +                   min( 0d0, max( alphapl,-alphapl*r) )
                  rhside(ip,jp,kp) = rhside(ip,jp,kp) - 0.5d0 *
     +                               vsum(ip+ishift,jp+jshift,kp+kshift,
     +                                    alphad) *
     +                               xlim * ( phi0 - phi2 )
                else
                  phi2 = solut(ip+ishift+2*dx(1),jp+jshift+2*dx(2),
     +                         kp+kshift+2*dx(3))
                  r  = ( phi1 - phi0 + 1d-10) / ( phi2 - phi1 + 1d-10 )
                  fact = min( mbound,
     +                        0.5d0*(1d0+kappa)*r + 0.5d0*(1d0-kappa) )
                  xlim = max( 0d0, min((2d0+alphapl)*r,fact) ) +
     +                   min( 0d0, max( alphapl,-alphapl*r) )
                  rhside(ip,jp,kp) = rhside(ip,jp,kp) + 0.5d0 *
     +                               vsum(ip+ishift,jp+jshift,kp+kshift,
     +                                    alphad) *
     +                               xlim * ( phi2 - phi1 )

                end if
              end do
            end do
          end do

        else if ( ichoisdis.eq.11 ) then

c       --- ichoisdis = 11, Symmetric PL-kappa limiter

          do kp = iarea(1,3)+1, iarea(2,3)-1
            do jp = iarea(1,2)+1, iarea(2,2)-1
              do ip = iarea(1,1)+1, iarea(2,1)-1

c             --- Compute the flux at the integration point and transport
c                 quantities in centroid and shifted points

                flux = veloc(ip+ishift,jp+jshift,kp+kshift,betad)
                phi0 = solut(ip+ishift,jp+jshift,kp+kshift)
                phi1 = solut(ip+ishift+dx(1),jp+jshift+dx(2),
     +                       kp+kshift+dx(3))

c               --- update right-hand side using the defect correction
c                   technique

                if ( flux.gt.0d0 ) then
                  phi2 = solut(ip+ishift-dx(1),jp+jshift-dx(2),
     +                         kp+kshift-dx(3))
                  r = ( phi1 - phi0 + 1d-10) / ( phi0 - phi2 + 1d-10 )
                  fact = min( mbound,
     +                        0.5d0*(1d0+kappa)*r + 0.5d0*(1d0-kappa) )
                  fact = min( fact,
     +                        0.5d0*(1d0-kappa)*r + 0.5d0*(1d0+kappa) )
                  xlim = max( 0d0, min(mbound*r,fact) )
                  rhside(ip,jp,kp) = rhside(ip,jp,kp) - 0.5d0 *
     +                               vsum(ip+ishift,jp+jshift,kp+kshift,
     +                                    alphad) *
     +                               xlim * ( phi0 - phi2 )
                else
                  phi2 = solut(ip+ishift+2*dx(1),jp+jshift+2*dx(2),
     +                         kp+kshift+2*dx(3))
                  r  = ( phi1 - phi0 + 1d-10) / ( phi2 - phi1 + 1d-10 )
                  fact = min( mbound,
     +                        0.5d0*(1d0+kappa)*r + 0.5d0*(1d0-kappa) )
                  fact = min( fact,
     +                        0.5d0*(1d0-kappa)*r + 0.5d0*(1d0+kappa) )
                  xlim = max( 0d0, min(mbound*r,fact) )
                  rhside(ip,jp,kp) = rhside(ip,jp,kp) + 0.5d0 *
     +                               vsum(ip+ishift,jp+jshift,kp+kshift,
     +                                    alphad) *
     +                               xlim * ( phi2 - phi1 )

                end if
              end do
            end do
          end do

        end if

      end if

      end
