      subroutine isco2get ( iblock, iface, isubface, ni, nj, nvirtual,
     +                      nblocks, nfaces, maxnsubf, subfaces,
     +                      coor, vco )
c ======================================================================
c
c        programmer   Erik Brakkee
c        version 1.0    date 30-11-1995 Guus: extracted from iscoget
c
c
c   copyright (c) 1995-1995  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Get the coordinates from a certain subface of a certain block that are
c     transported to neighbouring blocks into an intermediate representation.
c     These coordinates are numbered in the increasing i and j directions.
c     Coordinates are numbered in the tangential direction first, then
c     in a direction normal to the interface. The x coordinates occur
c     first then the y coordinates.
c     The intermediate representation may be treated as a multi-dimensional
c     array as follows:
c     vco( 1:nvirtual, 1:ndim, 1:number_of_coordinates_in_tangential_dir)
c **********************************************************************
c
c                       KEYWORDS
c
c   topology
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iblock, iface, isubface, maxnsubf, nblocks,
     +        nfaces, ni, nj, nvirtual
      double precision coor(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,2),
     +                 vco(nvirtual,2,*)
      integer subfaces(1:nblocks, 1:nfaces, 1:maxnsubf, *)

c     coor      i    reserved
c     iblock    i    Block sequence number
c     iface     i    Face number
c     isubface  i    Subface number
c     maxnsubf  i    Maximum number of subfaces encountered on all subfaces
c     nblocks   i    Number of blocks
c     nfaces    i    Number of faces
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     subfaces  i    reserved
c     vco       o    Work array in which the co-ordinates to be copied are
c                    stored
c                    vco has dimension: vco(1:nvirtual, 1:ndim, 1:nijk+1)
c                    with nijk max(ni,nj,nk)
c                    Array vco(i,m,n) contains the differences between points
c                    at a distance from the boundary and the corresponding
c                    points on the boundary.
c                    i:  the distance normal to the subface
c                    m:  refers tot the x (m=1), y (m=2), or z (m=3)
c                        co-ordinate
c                    n:  the distance along the subface
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i1, i2, ihigh, ii, ilow, inorm, j1, j2, jj, k,
     +        ishift, jshift

c     i1        Lower index number along first direction of a face or edge
c     i2        Upper index number along first direction of a face or edge
c     ihigh     Upper index of subface
c     ii        Loop variable
c     ilow      Lower index of subface
c     inorm     Loop variable
c     ishift    Shift in i-direction
c     j1        Lower index number along second direction of a face
c     j2        Upper index number along second direction of a face
c     jj        Loop variable
c     jshift    Shift in j-direction
c     k         General loop variable
c **********************************************************************
c
c                       I/O
c
c      none
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     none
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c     Compute the i and j bounds for the co-ordinates at the boundary
c     Put the co-ordinate differences into vco
c
c     The reason that vco contains the differences rather than the actual
c     values is only because periodical boundary conditions may be
c     treated by coupling of two sides of the boundary onto each other
c     As a consequence it is necessary that copying must be performed
c     such that cells at both sides of the region are again periodical
c     This can be done by computing the distances rather than by copying
c
c     Actual method:
c
c
c     -------------------------------------------------------------
c     |     |     |     |     |     |     |     |     |     |     |
c     |     |     |     |     |     |     |     |     |     |     |
c     |     |     |     |     |     |     |     |     |     |     |
c     |     |     |     |     |     |     |     |     |     |     |
c     |     |     |     |     |     |     |     |     |     |     |
c     |     |     |     |     |     |     |     |     |     |     |
c     |     |     |     |     |     |     |     |     |     |     |
c     |     |     |     |     |     |     |     |     |     |     |
c     |     |     |     |     |     |     |     |     |     |     |
c     |     |     |     |     |     |     |     |     |     |     |
c     |     |     |     |     |     |     |     |     |     |     |
c     -------------------------------------------------------------
c     1     2     3
c
c     As an example we consider the left boundary
c
c     vco(i,..) is filled with the differences:
c
c        coor(i+1,...) - coor(1,...)
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
c     --- Compute the i and j bounds for the co-ordinates at the boundary

      ilow = subfaces(iblock, iface, isubface, 1)
      ihigh = subfaces(iblock, iface, isubface, 2) + 1
      if ( iface.eq.ileft ) then

         ii = 1
         j1 = ilow
         j2 = ihigh

      else if ( iface.eq.iright ) then

         ii = ni+1
         j1 = ilow
         j2 = ihigh

      else if ( iface.eq.idown ) then

         i1 = ilow
         i2 = ihigh
         jj = 1

      else

         i1 = ilow
         i2 = ihigh
         jj = nj+1

      end if

c     --- fill vco with difference of coordinates to be transported
c         and boundary values of coor (needed for periodic boundary
c         conditions)

      do inorm = 1, nvirtual

         if ( iface.eq.ileft ) then

            ishift = inorm

         else if ( iface.eq.iright ) then

            ishift = -inorm

         else if ( iface.eq.idown ) then

            jshift = inorm

         else

            jshift = -inorm

         end if

         if ( iface.eq.iright .or. iface.eq.ileft ) then

c        --- right or left face, loop over j

            do k = 1, 2
               do jj = j1, j2
                  vco(inorm,k,jj-j1+1) = coor(ii+ishift,jj,k) -
     +                                   coor(ii,jj,k)
               end do
            end do

         else

c        --- bottom or top face, loop over i

            do k = 1, 2
               do ii = i1, i2
                  vco(inorm,k,ii-i1+1) = coor(ii,jj+jshift,k) -
     +                                   coor(ii,jj,k)
               end do
            end do

         end if

      end do
      end
