      subroutine isdinvu3 ( bresul, nusol, ndimso, prec, nprec,
     +                      initer )
c ======================================================================
c
c        programmer       Kees Vuik
c        version 1.2    date 07-07-1998 Kees V: nusol, nsolut cleaned
c        version 1.1      date 23-08-1995  correction of the loop
c                         at the upper boundary
c        version 1.0      date 04-10-1994  extracted from old ISNaS
c                         subroutine dinvu3.
c
c   copyright (c) 1994-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Multiplication of b by U, the preconditioning matrix given in
c     prec.
c     In this case we obtain:
c                 -1
c            b = U  * b.
c     The array prec should be filled by isdmlu3.f. This subroutine
c     contains compiler directives to run in vector speed on the
c     Convex.
c     Remark: In this version we use more dimensional arrays for the
c     preconditioning and the original/result vector. This is not
c     necessary, since this subroutine is only used for the pressure
c     and transport equations, where ndimso = 1. We have done this
c     to be able to generalise this preconditioner for the momentum
c     equation and/or the coupled approach (in the future).
c **********************************************************************
c
c                       KEYWORDS
c
c     linear_solver
c     preconditioner
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer          initer(*), nusol, ndimso, nprec
      double precision prec(1:nusol,1:ndimso,1:nprec),
     +                 bresul(nusol,ndimso)

c     bresul   i/o   The result vector
c     initer    i    reserved
c     ndimso    i    Integer indicating the dimension of the space in
c                    which the problem must be solved (ndimso = 1 or ndim)
c     nprec     i    Maximum number of diagonals in preconditioning matrix
c     nusol     i    Number of unknowns for one degree of freedom in
c                    one block
c     prec      i    The preconditioning matrix
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer   i,j,ii,nx,ny,jjmin,jjmax

c     i         General loop variable
c     ii        Loop variable
c     j         General loop variable
c     jjmax     Loop bound
c     jjmin     Loop bound
c     nx        Number of grid x points on finest grid
c     ny        Number of grid y points on finest grid
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c     Multiplication of b by U preconditioning matrix given in prec.
c     In this case we obtain:
c                 -1
c            b = U  b.
c
c     The array prec should be filled by subroutine isdmlu3.f. This
c     subroutine is closely linked to isdinvl3.f, where the lower
c     triangular system is solved.
c
c     In this subroutine the upper triangular system is solved in such
c     a way that it runs in vector speed on the Convex C3840 vector-
c     computer. The literature where this approach is described is:
c
c     Literature
c
c     C.C. Ashcraft and R.G. Grimes,
c     On vectorizing incomplete factorization and SSOR preconditioners,
c     SIAM. J. Sci. Stat. Comput. 9 pp. 122--151 (1988).
c
c     H. A. van der Vorst,
c     High performance preconditioning,
c     SIAM. J. Sci. Stat. Comput. 10 pp.1174-1185 (1989).
c
c     C. Vuik,
c     Fast iterative solvers for the discretized incompressible Navier-
c     Stokes equations,
c     TUD Report 93-98,
c
c
c     Remark: At this moment only the pressure and transport equations
c     are allowed.
c
c     Below we give the vectorized code for the main do-loops.
c     We note that due to the 9-point stencil the vectorization
c     is more difficult than for a 5-point stencil. Consider the
c     following two-dimensional grid:
c
c
c                +  +  +  +  +
c
c                o  +  +  +  +
c
c                o  *  *  +  +
c         ^
c         |      o  *  *  *  o
c
c        i2      o  o  o  o  o
c
c                   i1 ->
c
c     * 9-point stencil, o value of b unknown, + value of b known.
c
c     We describe the position by two counters i1 along the x1-axis,
c     and i2 along the x2-axis. It follows that the points on the
c     diagonals  i1 + 2*i2 = constant, can be calculated independently
c     (so in vector speed). We distinguish odd and even diagonals. The
c     ii-th odd diagonal is a line through the points (1,ii) and
c     (2*ii-1,1) and is considered in the first loop (constant = 2*ii+1),
c     the ii-th even diagonal is a line through the points (2,ii) and
c     (2*ii,1) and is considered in the second loop (constant = 2*ii+2).
c     The number of odd (even) diagonals is bounded by ii <= ny+(nx-1)/2.
c     The other bounds, jjmin and jjmax, come from the fact that the
c     diagonals are only considered in the rectangular block:
c     1 <= i1 <= nx, and 2  <= i2 <= ny (note that i2 = ny is considered
c     separately). Finally we explain the formula
c
c                 i = 2*ii+1-2*j+(j-1)*nx                            (1)
c
c     in the second loop (the explanation for the first loop is
c     comparable). First we have the formula
c
c                 i1 + 2*i2 = 2*ii + 1
c
c     so i1 = 2*ii + 1 - 2*i2. Secondly the 2-dimensional vector is
c     stored as a one dimensional array, where
c
c                 i = i1 + (i2-1)*nx                                 (2)
c
c     Combination of (1) and (2) leads to
c
c                 i = 2*ii+1-2*i2+(i2-1)*nx,
c
c     substitute j = i2 and one gets the correct formula.
c
c
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      nx = initer(12)+1
      ny = initer(13)+1

c     --- First we fill the points on the upper horizontal boundary.

      bresul(nusol,ndimso) = bresul(nusol,ndimso)
     +                        *prec(nusol,ndimso,1)

      do i = nusol-1, nusol-nx+1, -1
         bresul(i,ndimso) = (bresul(i,ndimso)
     +                      -prec(i,ndimso,6)*bresul(i+1,ndimso))
     +                      *prec(i,ndimso,1)
      end do

c     --- Thereafter the first two points are filled on the next horizontal
c         line.

      i = (ny-1)*nx
      bresul(i,ndimso) = (bresul(i,ndimso)
     +                   -prec(i,ndimso,7)*bresul(i+nx-1,ndimso)
     +                   -prec(i,ndimso,8)*bresul(i+nx  ,ndimso))
     +                   *prec(i,ndimso,1)

      if(nx.eq.2*(nx/2)) then
         i = (ny-1)*nx-1
         bresul(i,ndimso) = (bresul(i,ndimso)
     +                      -prec(i,ndimso,6)*bresul(i+1   ,ndimso)
     +                      -prec(i,ndimso,7)*bresul(i+nx-1,ndimso)
     +                      -prec(i,ndimso,8)*bresul(i+nx  ,ndimso)
     +                      -prec(i,ndimso,9)*bresul(i+nx+1,ndimso))
     +                      *prec(i,ndimso,1)
      end if

      do ii = ny-2+(nx-1)/2, 1, -1

         jjmin = max(1,ii+1-nx/2)
         jjmax = min(ii,ny-1)

c$dir no_recurrence

         do j = jjmin, jjmax

c        --- the even diagonals

            i = 2*ii+2-2*j+(j-1)*nx
            bresul(i,ndimso) = (bresul(i,ndimso)
     +                         -prec(i,ndimso,6)*bresul(i+1   ,ndimso)
     +                         -prec(i,ndimso,7)*bresul(i+nx-1,ndimso)
     +                         -prec(i,ndimso,8)*bresul(i+nx  ,ndimso)
     +                         -prec(i,ndimso,9)*bresul(i+nx+1,ndimso))
     +                         *prec(i,ndimso,1)
         end do

         jjmin = max(1,ii-(nx-1)/2)
         jjmax = min(ii,ny-1)

c$dir no_recurrence

         do j = jjmin, jjmax

c        --- the odd diagonals

            i = 2*ii+1-2*j+(j-1)*nx
            bresul(i,ndimso) = (bresul(i,ndimso)
     +                         -prec(i,ndimso,6)*bresul(i+1   ,ndimso)
     +                         -prec(i,ndimso,7)*bresul(i+nx-1,ndimso)
     +                         -prec(i,ndimso,8)*bresul(i+nx  ,ndimso)
     +                         -prec(i,ndimso,9)*bresul(i+nx+1,ndimso))
     +                         *prec(i,ndimso,1)
         end do
      end do
      end
