      subroutine isgmres ( nusol2, amat, rhsd, usol, work, solutv,
     +                     infmat, prec, resid, initer, reiter,
     +                     iinmtvc )
c ======================================================================
c
c        programmer     Kees Vuik
c        version 1.8    date 07-07-1998 Kees V: nusol, nsolut cleaned
c        version 1.7    date 19-11-1997 Kees K. iinmtvc added
c        version 1.6    date 21-10-1997 Kees K. iinmtvc added
c        version 1.5    date 21-01-1997 Kees V. adaptation of printing
c        version 1.4    date 31-01-1996 Guus: call print routine only when
c                                       necessary
c        version 1.3    date 03-10-1995 Kees K. solutv added
c        version 1.3    date 28-08-1995 Kees V. printing adapted
c        version 1.2    date 25-04-1995 Kees V. warning 1007 adapted
c        version 1.1    date 04-04-1995 Guus replace mtvc1 by ismatvec
c        version 1.0    date 09-01-1995
c
c
c   copyright (c) 1995-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     The GMRES method is used to solve an unsymmetric system of linear
c     equations of the shape A x = f.
c **********************************************************************
c
c                       KEYWORDS
c
c     linear_solver
c     preconditioner
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer          nusol2, infmat(*), initer(*), iinmtvc(*)
      double precision amat(nusol2,*), rhsd(nusol2), usol(nusol2),
     +                 work(nusol2,*), prec(nusol2,*), resid(*),
     +                 reiter(*), solutv(*)

c     amat      i    The coefficients matrix used in the linear system
c     iinmtvc   i    reserved
c     infmat    i    Integer information for the matrix
c     initer   i/o   reserved
c     nusol2    i    Number of scalar unknowns for the solvers
c                    nusol2 = nusol * ndimso
c     prec      i    The preconditioning matrix
c     reiter    i    reserved
c     resid     o    Array containing the norm of the residuals
c     rhsd      i    Vector containing the right-hand side vector of the
c                    system of equations.
c     solutv   i/o   Work array for solut with virtual points
c     usol     i/o   Solution vector of length n. On input the array should
c                    contain a starting vector. At output the array contains
c                    the last iterate, which is an approximation to the
c                    solution of the system.
c     work      i    Work array
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      logical          start
      integer          inn, itmax, inmax, iter, j, k, iprinlin,
     +                 job, lda, ldu, ldv, ns, info, nmaxsi, nints,
     +                 nreal
      double precision rnorm, thulp, epslin, hh(201,200),
     +                 cosgm(200), singm(200), rs(201), ddot, dnrm2,
     +                 eps1, eps2, eps3, es(50), singval(50),
     +                 hhulp(50,50),us(50,50), vs(50,50), eswork(50),
     +                 bnorm, r0norm

c     bnorm     Contains the norm of the righthandside vector after
c               preconditioning.
c     cosgm     Array determining the plane rotations which transform the
c               upper Hessenberg-matrix into an upper triangular matrix.
c     eps1      Determine the accuracy of the final approximation. The
c               termination criterion is based upon
c               ||b-Ax || <  eps1 + eps2 * (b-Ax ) .
c                     k                         0
c     eps2      Determine the accuracy of the final approximation. The
c               termination criterion is based upon
c               ||b-Ax || <  eps1 + eps2 * (b-Ax ) .
c                     k                         0
c     eps3      If eps3 is greater then zero, then the GMRES method
c               stops as K(A)*||b-Ax ||/||b|| < eps3.
c               An estimate of K(A) is made by the calculation of the
c               singular values of the Hessenberg matrix.
c     epslin    Required accuracy (including relative and absolute error)
c               in the linear solver.
c     es        Work array for subroutine isdsvdc.
c     eswork    Work array for subroutine isdsvdc.
c     hh        Array containing an upper Hessenberg-matrix of dimension
c               (i+1).
c     hhulp     This array is used to make a copy of the array hh.
c     info      Parameter used in subroutine isdsvdc.
c     inmax     Maximal value for inn.
c     inn       Loop variable for the inner iteration.
c     iprinlin  Control parameter indicating the amount of output.
c     iter      The current iteration number.
c     itmax     The maximum number of iterations to be performed in
c               the linear solver.
c     j         General loop variable
c     job       Parameter used in subroutine isdsvdc.
c     k         General loop variable
c     lda       Leading dimension of the array hhulp.
c     ldu       Leading dimension of the array us.
c     ldv       Leading dimension of the array vs.
c     nints     Number of integers in a message
c     nmaxsi    Maximal dimension of the matrix hh for which the
c               singular values are calculated.
c     nreal     Number of reals in a message
c     ns        Parameter used in subroutine isdsvdc.
c     r0norm    This is the norm of the initial residual, and it
c               is used in the calculation of the residual with
c               vector updates.
c     rnorm     Norm of the residual vector.
c     rs        Array containing the norms of the residuals from the inner
c               iterations.
c     singm     Array determining the plane rotations which transform the
c               upper Hessenberg-matrix into an upper triangular matrix.
c     singval   Array contains the singular values of hh.
c     start     Logical value indicating whether a restart is made.
c     thulp     Auxiliary variable.
c     us        Array containing the left singular vectors.
c     vs        Arrays containing the right singular vectors.
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     DAXPY     Computes a vector + a constant * other vector. This
c               is a BLAS routine.
c     DDOT      Computes an inner product. This is a BLAS routine.
c     DNRM2     Calculates the 2-norm of a given vector. This is
c               a BLAS routine.
c     DSVDC     Linpack subroutine to calculate singular values
c     ISDIAGMU  Computes the multiplication of a diagonal preconditioner
c               stored in array prec and a vector.
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERREAL  Put real value in error message
c     ISERRINT  Put integer value in error message
c     ISMATVEC  Computes the product of the sparse matrix, stored in amat,
c               and a vector.
c     ISPREFIN  Scales the residual if a postconditioning is used
c     ISPREMAT  Computes a matrix vector product combined
c               with a pre- or postconditioning matrix
c     ISPRESTR  Scales the residual if a preconditioning is used
c     ISPRINT   ISNaS general print subroutine
c     ISWARSUB  Print warnings
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c    1002   work space is too small
c    1007   The required accuracy is too small.
c    1008   The required restart value is too large
c **********************************************************************
c
c                       PSEUDO CODE
c
c     The GMRES method is used to solve an unsymmetric system of linear
c     equations of the shape A x = f.
c     Starting from an initial vector, in each iteration a vector is sought
c     which minimizes the residual vector A.x-f in a certain Krylov-subspace.
c     The process restarts after a fixed number of iteration, with the latest
c     approximation as a new starting vector.
c
c     Reference:
c
c     Y. Saad & M.H. Schultz,
c     GMRES: a generalized minimal residual algorithm for solving non
c     symmetric linear systems.
c     SIAM J. Sci. Stat. Comput. 7, 856-869, (1986).
c
c
c     Extra features:
c     ---------------
c
c     1. If eps3 > 0 an estimate of the condition number of A is made.
c        The calculation of the singular value decomposition is done by
c        a call to the Linpack-subroutine dsvdc. The singular values are
c        used to estimate K(A), the 2-norm condition number of A. This
c        is used in the stopping criterion
c
c                  K(A)*||b-Ax ||/||b|| < eps3.
c                             k
c
c        A warning is given if
c
c                  1000*epsmac > eps3*||b||/(K(A)*||r0||,
c
c        in this case eps3 is to small with respect to the machine
c        precision.
c
c     2. In the case of a restart the residual vector is calculated by
c        vector updates instead of the formula r  = b-Ax
c                                               k       k
c        see: Saad & Schultz p.863
c
c     3. The Eisenstat trick is implemented for the ILUD preconditioning.
c
c        References:
c
c        S.C. Eisenstat
c        Efficient implementation of a class of preconditioned conjugate
c        gradient methods
c        SIAM J. Sci. Stat. Comp. 2 pp. 1-4 (1981)
c
c        H. A. van der Vorst
c        Iterative solution for certain sparse linear systems with non-
c        symmetric matrix arising from PDE-problems
c        Journal of Computational Physics 44 pp. 1-19 (1981)
c
c     4. In order to avoid underflow in the solution of a lower triangular
c        matrix in the calculation of the first residual, we take as start
c        solution usol = |rhsd|*10^-34.
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      call iseropen ( 'isgmres' )

c     --- Initialization of solver parameters

      itmax     = initer(10)
      iprinlin  = initer(2)
      nints     = initer(19)
      eps1      = reiter(1)
      eps2      = reiter(2)
      eps3      = reiter(3)
      start     = .true.
      initer(4) = 0

c     --- Initialization of constants for the call of dsvdc

      lda = 50
      ldu = lda
      ldv = lda
      job = 0
      nmaxsi = 10

c     --- Determination of the maximum number of inner iterations.

      if ( initer(6).le.0 .or. initer(6).ge.200 ) then
         if ( iprinlin.ge.1 .and. initer(6).gt.200 ) then
            call iserrint ( initer(6), 1)
            call iswarsub(1008,1,0,0)
         end if
         inmax = 200
      else
         inmax = initer(6)
      end if

c     --- If initer(1) equals 2 (ILUD preconditioner), then the linear
c         system is scaled by a diagonal matrix, such that diag(L)=diag(U)=D=I.

      if ( initer(1).eq.2 ) then
         call isdiagmu ( nusol2, rhsd, prec(1,2) )
      end if

c     --- In order to avoid underflow in the solution of a lower
c         triangular system, we take as start solution
c         usol = usol + ||rhsd||*10^(-34)

      if ( abs(initer(1)).eq.2 .or. abs(initer(1)).eq.3 ) then
         rnorm = dnrm2 ( nusol2, rhsd, 1 )*1d-34
           if ( rnorm.gt.0 ) then
             do j = 1, nusol2
                usol(j) = usol(j)+rnorm
             end do
           end if
      end if

      rnorm  = 0
      epslin = 0
      call isprestr ( nusol2, rhsd , work(1,2), amat, infmat,
     +                prec, initer )
      bnorm = dnrm2 ( nusol2, work(1,2), 1 )

c     --- Main iteration loop.

      do iter = 1, itmax

c     --- Compute residual vector and its norm in case of restart.

         if ( start ) then
            inn = 0
            if ( rnorm.lt.epslin .or. iter.eq.itmax
     +           .or. iter.eq.1 ) then

              call ismatvec(amat, usol, solutv, work, infmat, initer,
     +                       iinmtvc)

               if ( initer(1).eq.2 ) then
                  do j = 1, nusol2
                     work(j,2) = rhsd(j)-work(j,1)-2*usol(j)
                  end do
               else
                  do j = 1, nusol2
                     work(j,2) = rhsd(j)-work(j,1)
                  end do
               end if
               call isprestr ( nusol2, work(1,2), work, amat, infmat,
     +                         prec, initer )
            end if
            rnorm  = dnrm2 ( nusol2, work, 1 )
            rs(1)  = rnorm
            r0norm = rnorm
            resid(iter) = rnorm

c           --- Compute required accuracy.

            if ( iter.eq.1) then
               epslin = eps1 + eps2 * rnorm
               thulp  = 1000*epsmac*bnorm
               if ( epslin.lt.thulp .and. rnorm.gt.0 ) then
                  if ( iprinlin.ge.1 ) then
                     call iserreal ( thulp, 1)
                     call iserreal ( epslin, 2)
                     call iswarsub(1007,0,2,0)
                  end if
                  epslin = thulp
               end if
            end if

c           --- Filling of some constants if eps3 > 0

            if ( iter.eq.1 .and. eps3.gt.0 ) then
               epslin = eps3*bnorm
               nmaxsi = min(nmaxsi, inmax)
            end if

c           --- Check for convergence.

            if (rnorm.le.epslin) then
               initer(3)=0
               goto 500
            end if

            start = .false.
            do j = 1, nusol2
               work(j,1) = work(j,1)/rnorm
            end do
         end if

         initer(4) = iter

c        --- Inner iteration step, apply modified Gram-Schmidt to form a
c            new column of hh.

         inn = inn+1
         call ispremat ( nusol2, work(1,inn), work(1,inn+1), amat,
     +                   infmat, prec, work(1,inn+2), solutv, initer,
     +                   iinmtvc )
         do j = 1, inn
            thulp = ddot ( nusol2, work(1,j), 1, work(1,inn+1), 1 )
            hh(j,inn) = thulp
            call daxpy ( nusol2, -thulp, work(1,j), 1,
     +                   work(1,inn+1), 1)
         end do
         hh(inn+1,inn) = dnrm2 ( nusol2, work(1,inn+1), 1 )

c        --- Perform plane rotations on new column.

         if ( inn.gt.1 ) then
            do k = 2, inn
               thulp = hh(k-1,inn)
               hh(k-1,inn) =  cosgm(k-1)*thulp + singm(k-1)*hh(k,inn)
               hh(k,inn)   = -singm(k-1)*thulp + cosgm(k-1)*hh(k,inn)
            end do
         end if

c        --- Calculate new plane rotation.

         thulp = sqrt(hh(inn,inn)**2 + hh(inn+1,inn)**2)
         if ( thulp.eq.0.0d0 ) thulp = epsmac
         cosgm(inn)  = hh(inn,inn)/thulp
         singm(inn)  = hh(inn+1,inn)/thulp
         rs(inn+1)   = -singm(inn)*rs(inn)
         rs(inn)     = cosgm(inn)*rs(inn)
         hh(inn,inn) = cosgm(inn)*hh(inn,inn) + singm(inn)*hh(inn+1,inn)

c        --- Determine residual norm.

         rnorm = abs(rs(inn+1))
         resid(iter+1) = rnorm

c        --- If eps3 > 0 then a call to dsvdc is made.

         if ( iter.eq.nmaxsi .and. eps3.gt.0 ) then
            ns = inn
            do j = 1, ns
               do k = 1, j
                  hhulp(k,j) = hh(k,j)
               end do
               do k = j+1, ns
                  hhulp(k,j) = 0d0
               end do
            end do
            call dsvdc ( hhulp, lda, ns, ns, singval, es, us, ldu,
     +                   vs, ldv, eswork, job, info)
            epslin = eps3*bnorm*singval(ns)/singval(1)

c           --- singval(1)/singval(ns) is an estimate of cond(A) after
c               nmaxsi iterations.

            thulp = 1000*epsmac*bnorm
            if ( epslin.lt.thulp ) then
               epslin = thulp
               if ( iprinlin.ge.1 ) then
                  call iserreal ( thulp, 1)
                  call iserreal ( epslin, 2)
                  call iswarsub(1007,0,2,0)
               end if
            end if
         end if

c        --- Check for termination of inner iterations.

         if ( inn.eq.inmax .or. rnorm.lt.epslin .or.
     +       iter.eq.itmax ) then

c        --- Compute new solution. First solve upper triangular system.

            rs(inn) = rs(inn)/hh(inn,inn)
            do k = inn-1, 1, -1
               thulp = rs(k)
               do j = k+1, inn
                  thulp = thulp-hh(k,j)*rs(j)
               end do
               rs(k) = thulp/hh(k,k)
            end do

c           --- Form linear combinations of residuals to get solution.

            if ( initer(1).eq.2 .or. initer(1).lt.0 ) then
               do j = 1, nusol2
                  work(j,inn+2) = rs(1)*work(j,1)
               end do
               do j = 2, inn
                  call daxpy ( nusol2, rs(j), work(1,j), 1,
     +                         work(1,inn+2), 1)
               end do
               call isprefin ( nusol2, work(1,inn+2), work(1,inn+3),
     +                         amat, infmat, prec, initer )
               call daxpy ( nusol2, 1d0, work(1,inn+3), 1,
     +                      usol, 1 )
            else
               do j = 1, inn
                  call daxpy ( nusol2, rs(j), work(1,j), 1,
     +                         usol, 1 )
               end do
            end if

            if ( inn.eq.inmax .and. rnorm.ge.epslin ) then

c           --- Calculate the residual with vector updates instead
c               of the formula r = b - Ax
c                               k        k
c               compute R*y

               do j = 1, nusol2
                  work(j,inn+1) = work(j,inn+1)/hh(inn+1,inn)
               end do
               do j = 1, inn
                  rs(j) = hh(j,j)*rs(j)
                  do k = j+1, inn
                     rs(j) = rs(j)+hh(j,k)*rs(k)
                  end do
               end do

c              --- T
c                  --- compute Q *R*y

               rs(inn+1) = 0d0
               do j = inn+1, 2, -1
                  thulp = rs(j-1)
                  rs(j-1) = cosgm(j-1)*thulp-singm(j-1)*rs(j)
                  rs(j)   = singm(j-1)*thulp+cosgm(j-1)*rs(j)
               end do

c              --- T
c                  --- compute beta*e - Q *R*y
c                  1

               do j = 1, inn+1
                  rs(j) = -rs(j)
               end do
               rs(1) = r0norm+rs(1)

c              --- compute r

               do j = 1, nusol2
                  work(j,1) = rs(1)*work(j,1)
               end do
               do j = 2, inn+1
                  call daxpy ( nusol2, rs(j), work(1,j), 1,
     +                         work, 1 )
               end do
            end if

c           --- Set restart.

            start = .true.
         else
            do j = 1, nusol2
               work(j,inn+1) = work(j,inn+1)/hh(inn+1,inn)
            end do
         end if

c        --- End of main iteration loop.

      end do

c     --- The iteration process terminates because the maximum number of
c         iterations is reached. Set error indicator.

      initer(3) = 3

c     --- Normal termination of the iteration process.

500   continue

c     --- If initer(1) equals 2 (ILUD preconditioner), then the linear
c         system is scaled by a diagonal matrix, such that diag(L)=diag(U)=D=I.
c         Below the right hand side is scaled back

      if ( initer(1).eq.2 ) then
         call isdiagmu ( nusol2, rhsd, prec )
      end if

c     --- Printing of linear solver information.

      if ( iprinlin.ge.1 .or. iprinlin.eq.0 .and. initer(3).ne.0 ) then

         nreal = initer(4)+1

c        --- Test if the amount of information contained in reiter
c            is less than 100 reals. Otherwise a warning is given.
c            The memory is reserved in subroutine islsds02.

         if ( initer(20).gt.100) then
            call iswarsub ( 1002, 0, 0, 0 )
         end if

         do j = 1, min ( 100, initer(20) )
            resid(nreal+j) = reiter(j)
         end do

         nreal = nreal+min ( 100, initer(20) )

         call isprint( 108, nints, initer, nreal, resid, 1, ' ')
      end if

      call iserclos ( 'isgmres' )

      end
