      subroutine ismatvec ( matrix, solut, solutv, rhside, infmat,
     +                      initer, iinmtvc )
c ======================================================================
c
c        programmer     Kees Kassels
c        version 1.6    date 06-08-1998 Guus: new calls to ismtvc21/2
c        version 1.5    date 08-07-1998 Kees V: nusol, nsolut cleaned
c        version 1.4    date 03-11-1997 Kees K. array iinmtvc added,
c                                       for periodic b.c. transport
c                                       routines for multi block used
c        version 1.3    date 31-10-1997 Kees V. changes for uncoupled
c        version 1.2    date 05-10-1995 work array solutv added 3D
c        version 1.1    date 03-10-1995 work array solutv added
c        version 1.0    date 04-04-1995 (Renamed subroutine mtvc1)
c
c   copyright (c) 1995-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     This subroutine performs the matrix vector multiplication for the
c     equations arising in ISNaS
c       A x = b, where b and x have the same structure
c       In one special case this subroutine is used to compute the correction
c       of the velocity where the pressure difference is given.
c       So in fact only one part of the matrix-vector multiplication is
c       carried out, and solution and right-hand side have a different structure
c       To recognise this special event a negative value of itypmat (-3)
c       is used
c **********************************************************************
c
c                       KEYWORDS
c
c      multiplication
c      matrix
c      vector
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer infmat(*), initer(*), iinmtvc(*)
      double precision matrix(*), rhside(*), solut(*), solutv(*)

c     iinmtvc i    reserved
c     infmat  i    Integer information for the matrix
c     initer i/o   reserved
c     matrix  i    reserved
c     rhside  o    reserved
c     solut   i    reserved
c     solutv  i    Work array for solut with virtual points
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisblank'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ni, nj, nk, itypmat, ipinfmat, nconct, nusol, ndegfd,
     +        ndim, iperiod
      integer iareanv(2,3), iareav(2,3), offset, idegfd, npointsnv,
     +        npointsv, nvirtual, isolv
      logical coupled

c     coupled   Logical which is true if the momentum equations
c               are coupled
c     iareanv   Bounds for array without virtual points
c     iareav    Bounds for array with virtual points
c     idegfd    Loop variable with respect to a given unknown
c     iperiod   Parameter indicating if periodical boundary conditions
c               are present per boundary
c               iperiod is a 3 digit number with the sequence 321
c               Each digit refers to 1 direction
c               The digits have the following meaning
c               0:  no periodical boundary conditions
c               1:  symmetrical periodical boundary conditions
c               2:  anti-symmetrical periodical boundary conditions
c     ipinfmat  Starting address in array infmat
c     isolv     Called by solver for periodic b.c. isolv=1 else isolv=0
c     itypmat   Type of matrix. Possible values
c               1:  Momentum matrix
c               2:  Pressure or transport matrix
c               3:  Gradient matrix
c     nconct    Number of connections in a row of the matrix
c     ndegfd    Number of degrees of freedom
c     ndim      Dimension of space (2 or 3)
c     ni        Number of non-virtual cells in the i-direction (one block)
c     nj        Number of non-virtual cells in the j-direction (one block)
c     nk        Number of non-virtual cells in the k-direction (one block)
c     npointsnv Number of points in the mesh without virtual points
c     npointsv  Number of points in the mesh with virtual points
c     nusol     Number of unknowns for one degree of freedom in
c               one block
c     nvirtual  Number of rows of virtual cells
c     offset    General offset factor
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISMBLK02  ?
c     ISMTVC01  Matrix-vector multiplication for momentum equations
c     ISMTVC03  Matrix-vector multiplication for gradient matrix (pressure)
c     ISMTVC21  Matrix-vector multiplication for 2D momentum equations
c               with periodical boundary conditions
c     ISMTVC22  Matrix-vector multiplication for 2D transport equations
c               with periodical boundary conditions
c     ISMTVC31  Matrix-vector multiplication for 3D momentum equations
c               with periodical boundary conditions
c     ISMTVC32  Matrix-vector multiplication for 3D transport equations
c               with periodical boundary conditions
c     ISSCOPY   Copy one array to another
c     ISVPPER3  Fill virtual point in solution for transport values 3D
c     ISVVPER3  Fill virtual point in solution for fluxes 3D
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- Compute general constants

      coupled = initer(29).eq.0

      iperiod = infmat(1)
      ni      = infmat(2)
      nj      = infmat(3)
      nk      = infmat(4)
      if ( nk.eq.0 ) then
         ndim = 2
      else
         ndim = 3
      end if
      nvirtual = 2

      itypmat = infmat(8)
      if ( iperiod.eq.0 ) then
         ipinfmat = infmat(13)
      else
         ipinfmat = infmat(12)
      end if
      nconct  = infmat(6)

      nusol  = (ni+1) * (nj+1) * (nk+1)

      if ( iperiod.gt.0 ) then

         iareanv(1,1) = 1
         iareanv(2,1) = ni+1
         iareanv(1,2) = 1
         iareanv(2,2) = nj+1
         if (ndim.eq.2) then
            iareanv(1,3) = 1
            iareanv(2,3) = 1
         else
            iareanv(1,3) = 1
            iareanv(2,3) = nk+1
         end if

         iareav(1,1) = 1-nvirtual
         iareav(2,1) = ni+1+nvirtual
         iareav(1,2) = 1-nvirtual
         iareav(2,2) = nj+1+nvirtual
         if (ndim.eq.2) then
            iareav(1,3) = 1
            iareav(2,3) = 1
         else
            iareav(1,3) = 1-nvirtual
            iareav(2,3) = nk+1+nvirtual
         end if
         npointsnv = (iareanv(2,1)-iareanv(1,1)+1)*
     +               (iareanv(2,2)-iareanv(1,2)+1)*
     +               (iareanv(2,3)-iareanv(1,3)+1)
         npointsv = (iareav(2,1)-iareav(1,1)+1)*
     +              (iareav(2,2)-iareav(1,2)+1)*
     +              (iareav(2,3)-iareav(1,3)+1)

      end if

      if ( itypmat.eq.1 .and. coupled ) then

c     --- matrix corresponds to coupled contravariant velocities

         if ( iperiod.gt.0 .and. ndim.eq.2 ) then

            do idegfd = 1, ndim

               offset = idegfd - 1

                  call isscopy ( iareanv(1,1),iareanv(2,1),
     +                           iareanv(1,2),iareanv(2,2),
     +                           iareanv(1,3),iareanv(2,3),
     +                           iareav(1,1) ,iareav(2,1),
     +                           iareav(1,2) ,iareav(2,2),
     +                           iareav(1,3) ,iareav(2,3),
     +                           1,iareanv,solut(1+offset*npointsnv),
     +                           solutv(1+offset*npointsv),1d0)

            end do

            isolv = 1
            call ismblk02( iinmtvc(1) , iinmtvc(2) , iinmtvc(3) ,
     +              iinmtvc(4) , iinmtvc(5) , iinmtvc(6) ,
     +              iinmtvc(7) , iinmtvc(8) , iinmtvc(9) ,
     +              iinmtvc(10), iinmtvc(11), iinmtvc(12),
     +              iinmtvc(13), isolv ,
     +              solutv, isolv, isolv, isolv,
     +              ibuffr(iinmtvc(14)),
     +              ibuffr(iinmtvc(15)),
     +              buffer(iinmtvc(16))  )

            call ismtvc21 ( matrix, solutv, rhside, ni, nj, nvirtual,
     +                      nconct, infmat(ipinfmat) )

         else if ( iperiod.gt.0 .and. ndim.eq.3 ) then

            do idegfd = 1, ndim

               offset = idegfd - 1

                  call isscopy ( iareanv(1,1),iareanv(2,1),
     +                           iareanv(1,2),iareanv(2,2),
     +                           iareanv(1,3),iareanv(2,3),
     +                           iareav(1,1) ,iareav(2,1),
     +                           iareav(1,2) ,iareav(2,2),
     +                           iareav(1,3) ,iareav(2,3),
     +                           1,iareanv,solut(1+offset*npointsnv),
     +                           solutv(1+offset*npointsv),1d0)

            end do

            call isvvper3 ( solutv, ni, nj, nk, nvirtual, iperiod )

            call ismtvc31 ( matrix, solutv, rhside, ni, nj, nk,
     +                      nvirtual, nconct, infmat(ipinfmat),
     +                      iperiod )

         else
            ndegfd = ndim
            call ismtvc01 ( matrix, solut, rhside, nusol, nconct,
     +                      ndegfd, infmat(ipinfmat) )
         end if

      else if ( itypmat.eq.2 ) then

c     --- matrix corresponds to G - matrix

         ndegfd = ndim
         call ismtvc03 ( matrix, solut, rhside, nusol, nconct,
     +                   ndegfd, infmat(ipinfmat) )

      else if ( abs(itypmat).eq.3 ) then

c     --- Matrix corresponds to pressure equation

         if ( iperiod.gt.0 .and. ndim.eq.2 ) then

            call isscopy ( iareanv(1,1),iareanv(2,1),
     +                     iareanv(1,2),iareanv(2,2),
     +                     iareanv(1,3),iareanv(2,3),
     +                     iareav(1,1) ,iareav(2,1),
     +                     iareav(1,2) ,iareav(2,2),
     +                     iareav(1,3) ,iareav(2,3),
     +                     1, iareanv , solut, solutv, 1d0)

            isolv = 1
            call ismblk02( iinmtvc(1) , iinmtvc(2) , iinmtvc(3) ,
     +                     iinmtvc(4) , iinmtvc(5) , iinmtvc(6) ,
     +                     iinmtvc(7) , iinmtvc(8) , iinmtvc(9) ,
     +                     iinmtvc(10), iinmtvc(11), iinmtvc(12),
     +                     iinmtvc(13), isolv ,
     +                     solutv, isolv, isolv, isolv,
     +                     ibuffr(iinmtvc(14)),
     +                     ibuffr(iinmtvc(15)),
     +                     buffer(iinmtvc(16)) )

            call ismtvc22 ( matrix, solutv, rhside, ni, nj, nvirtual,
     +                      nconct, infmat(ipinfmat) )

         else if ( iperiod.gt.0 .and. ndim.eq.3 ) then

            call isscopy ( iareanv(1,1),iareanv(2,1),
     +                     iareanv(1,2),iareanv(2,2),
     +                     iareanv(1,3),iareanv(2,3),
     +                     iareav(1,1) ,iareav(2,1),
     +                     iareav(1,2) ,iareav(2,2),
     +                     iareav(1,3) ,iareav(2,3),
     +                     1, iareanv  , solut, solutv, 1d0)

            call isvpper3 ( solutv, ni, nj, nk, nvirtual, iperiod )

            call ismtvc32 ( matrix, solutv, rhside, ni, nj, nk,
     +                      nvirtual, nconct, infmat(ipinfmat),
     +                      iperiod )

         else
            ndegfd = 1
            call ismtvc01 ( matrix, solut, rhside, nusol, nconct,
     +                      ndegfd, infmat(ipinfmat) )
         end if

      else if ( itypmat.ge.4 .or. itypmat.eq.1 ) then

c     --- matrix corresponds to transport equation or an uncoupled
c         momentum equation

         if ( iperiod.gt.0 .and. ndim.eq.2 ) then

            call isscopy ( iareanv(1,1),iareanv(2,1),
     +                     iareanv(1,2),iareanv(2,2),
     +                     iareanv(1,3),iareanv(2,3),
     +                     iareav(1,1) ,iareav(2,1),
     +                     iareav(1,2) ,iareav(2,2),
     +                     iareav(1,3) ,iareav(2,3),
     +                     1, iareanv , solut, solutv, 1d0)

            isolv = 1
            call ismblk02( iinmtvc(1) , iinmtvc(2) , iinmtvc(3) ,
     +                     iinmtvc(4) , iinmtvc(5) , iinmtvc(6) ,
     +                     iinmtvc(7) , iinmtvc(8) , iinmtvc(9) ,
     +                     iinmtvc(10), iinmtvc(11), iinmtvc(12),
     +                     iinmtvc(13), isolv ,
     +                     solutv, isolv, isolv, isolv,
     +                     ibuffr(iinmtvc(14)),
     +                     ibuffr(iinmtvc(15)),
     +                     buffer(iinmtvc(16)) )

            call ismtvc22 ( matrix, solutv, rhside, ni, nj, nvirtual,
     +                      nconct, infmat(ipinfmat) )

         else if ( iperiod.gt.0 .and. ndim.eq.3 ) then

            call isscopy ( iareanv(1,1),iareanv(2,1),
     +                     iareanv(1,2),iareanv(2,2),
     +                     iareanv(1,3),iareanv(2,3),
     +                     iareav(1,1) ,iareav(2,1),
     +                     iareav(1,2) ,iareav(2,2),
     +                     iareav(1,3) ,iareav(2,3),
     +                     1, iareanv  , solut, solutv, 1d0)

            call isvpper3 ( solutv, ni, nj, nk, nvirtual, iperiod )

            call ismtvc32 ( matrix, solutv, rhside, ni, nj, nk,
     +                      nvirtual, nconct, infmat(ipinfmat),
     +                      iperiod )

         else
            ndegfd = 1
            call ismtvc01 ( matrix, solut, rhside, nusol, nconct,
     +                      ndegfd, infmat(ipinfmat) )
         end if

      end if
      end
