      subroutine ismtvc24 ( gmat, prescorr, ustar, ni, nj,
     +                      nvirtual, gmctinv , coor, unfac, axisym)
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.4    date 08-03-1999 Kees K: bug fixed axisymmetric
c        version 1.3    date 02-02-1999 Kees K: axisymmetric
c        version 1.2    date 27-07-1998 Duncan: underrelaxation added
c                                       for iterative pressure correction
c                                       (period1,period2 removed)
c        version 1.1    date 19-11-1997 Kees K.:action for periodical
c                                       b.c. removed
c        version 1.0    date 17-07-1996
c
c
c   copyright (c) 1996-1999  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     This subroutine performs the of the matrix vector multiplication
c     f := f - (1-a) G p in 2D
c     where p is the pressure including virtual points,
c           g is the gradient matrix (no virtual points) and
c           f is the right-hand side (no virtual points) to be updated
c           a is an under relaxtion factor for iterative solution of the
c             pressure correction
c     Special axisymmetric case : devide correction by r
c **********************************************************************
c
c                       KEYWORDS
c
c      axisymmetric
c      multiplication
c      matrix
c      vector
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, gmctinv(0:3,2,6), axisym
      double precision gmat(ni+1,nj+1,2,6),
     +                 ustar(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,2),
     +                 coor (1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,2),
     +                 prescorr(1-nvirtual:ni+1+nvirtual,
     +                          1-nvirtual:nj+1+nvirtual),
     +                 unfac

c     axisym    i    In axisymmetric case axisym=1 else axisym=0
c     coor      i    reserved
c     gmat      i    Gradient matrix
c     gmctinv   i    Connection+row degree of freedom to column indices
c                    for gradient matrix
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     prescorr  i    Array containing the pressure correction
c     unfac     i    Under relaxation factor for the pressure correction
c     ustar    i/o   The intermediate velocity vector u*
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, ishift, j, jshift, l

c     i         General loop variable
c     ishift    Shift in i-direction
c     j         General loop variable
c     jshift    Shift in j-direction
c     l         General loop variable
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c     The matrix vector multiplication is defined by:  a   x  = f
c                                                       ij  j    i
c
c     Since the matrix has a different structure for both degrees of freedom
c     we have to perform different loops
c
c     Algorithm:
c
c     For l := 1, 6 do
c        ishift := gmctinv(1,1,l)
c        jshift := gmctinv(2,1,l)
c        For i := 1 (1) ni+1 do
c           For j := 1 (1) nj do
c              f(i,j,1) := f(i,j,1)+a(i,j,1,l)*
c                          x(i+ishift,j+jshiftt)
c
c     For l := 1, 6 do
c        ishift := gmctinv(1,2,l)
c        jshift := gmctinv(2,2,l)
c        For i := 1 (1) ni do
c           For j := 1 (1) nj+1 do
c              f(i,j,2) := f(i,j,2)+a(i,j,2,l)*
c                          x(i+ishift,j+jshift)
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c

      if ( axisym.eq.1) then

c     --- axisymmetric case deviding by  r

c         --- Loop over the connections

         do l = 1, 6

            ishift = gmctinv(1,1,l)
            jshift = gmctinv(2,1,l)
	    
c	    --- i starts here with 2, otherwise deviding by zero

            do j = 1, nj
               do i = 2, ni+1
                  ustar(i,j,1) = ustar(i,j,1) + gmat(i,j,1,l) *
     +                        (1d0-unfac) * prescorr(i+ishift,j+jshift)/
     +                       ( 5d-1 * ( coor(i,j,1)+
     +                         coor(i,j+1,1)))
               end do
            end do

            ishift = gmctinv(1,2,l)
            jshift = gmctinv(2,2,l)

            do j = 1, nj+1
               do i = 1, ni
                  ustar(i,j,2) = ustar(i,j,2) + gmat(i,j,2,l) *
     +                        (1d0-unfac) * prescorr(i+ishift,j+jshift)/
     +                       ( 5d-1 * (coor(i,j,1)+
     +                         coor(i+1,j,1)))
               end do
            end do

         end do

      else

c     --- Loop over the connections

         do l = 1, 6

            ishift = gmctinv(1,1,l)
            jshift = gmctinv(2,1,l)

            do j = 1, nj
               do i = 1, ni+1
                  ustar(i,j,1) = ustar(i,j,1) + gmat(i,j,1,l) *
     +                        (1d0-unfac) * prescorr(i+ishift,j+jshift)
               end do
            end do

            ishift = gmctinv(1,2,l)
            jshift = gmctinv(2,2,l)

            do j = 1, nj+1
               do i = 1, ni
                  ustar(i,j,2) = ustar(i,j,2) + gmat(i,j,2,l) *
     +                        (1d0-unfac) * prescorr(i+ishift,j+jshift)
               end do
            end do

         end do

      end if

      end
